<?php
if (!defined('WP_PLUGIN_DIR')) exit('No direct script access allowed');
/*
* Plugin Name: MonLabo
* Plugin URI: http://www.monlabo.org
* Description:  Simplifier la gestion d'un site web d'une unité de recherche
* Version: 2.7
* Author: Hervé SUAUDEAU (créateur) et Delphine RIDER (co-auteure)
* Text Domain: MonLabo
*/
function get_version() { return "2.7"; }

///////////////////////////////////////////////////////////////////////////////////////////
//                       PLUGIN CLASS DEFINITIONS
///////////////////////////////////////////////////////////////////////////////////////////

require_once( dirname(__FILE__) . '/includes/MonLabo_lib.php' );
require_once( dirname(__FILE__) . '/includes/MonLabo_persons.php' );
require_once( dirname(__FILE__) . '/includes/MonLabo_shortcodes_publications.php' );
require_once( dirname(__FILE__) . '/includes/MonLabo_shortcodes.php' );
require_once( dirname(__FILE__) . '/includes/MonLabo_db.php' );
require_once( dirname(__FILE__) . '/includes/constants.php' );
require_once( dirname(__FILE__) . '/includes/MonLabo_access_data.php' );
require_once( dirname(__FILE__) . '/includes/MonLabo_generate_HTML.php' );

/* Function to test if a column is present in a Mysql table
***********************************************************/
function is_column_exist_in_table($table_name, $column_name) {
    global $wpdb;
    $result = $wpdb->get_results("SHOW COLUMNS FROM `".$table_name."` WHERE FIELD='".$column_name."';");
    //$result = $wpdb->get_results("SELECT COUNT(*) AS C FROM information_schema.COLUMNS WHERE TABLE_NAME = '".$table_name."' AND COLUMN_NAME = '".$column_name."';");
    /*if (intval($result[0]->C) < 1 ) { //La colonne $column_name n'existe pas?
        return true;
    } else {
        return false;
    }*/
    if(count($result)>0) {
        return true;
    } else {
        return false;
    }
}

/* Function to test if a column has the right type in a Mysql table
******************************************************************/
function is_column_as_type_in_table($table_name, $column_name, $type) {
    global $wpdb;
    $results = $wpdb->get_results("SHOW COLUMNS FROM `".$table_name."` WHERE FIELD='".$column_name."';");
    foreach ($results as $key => $result) {
        if (strcmp ($result->Type, $type) === 0)
            return true;
    }
    return false;
}

/* main
*******/
if (!class_exists('MonLabo')) {
    class MonLabo {
        function  __construct() {
            add_action('init', array(&$this, 'MonLabo_init'));
            if ( is_admin() ) {
                require_once( dirname(__file__).'/admin/MonLabo_admin.php' );
            }
        }

        //hook into WP's admin_init action hook
        public function MonLabo_init()
        {
            $options=get_option( 'MonLabo_settings_group0' );
            if (isset($options["MonLabo_uses_members_and_groups"]) && (intval($options["MonLabo_uses_members_and_groups"])===1)) {
                add_shortcode( 'members_list', array('MonLabo_shortcodes', 'members_list') );
                add_shortcode( 'members_table', array('MonLabo_shortcodes', 'members_table') );
                add_shortcode( 'members_chart', array('MonLabo_shortcodes', 'members_chart') );
                add_shortcode( 'alumni_list', array('MonLabo_shortcodes', 'alumni_list') );
                add_shortcode( 'alumni_table', array('MonLabo_shortcodes', 'alumni_table') );
                add_shortcode( 'alumni_chart', array('MonLabo_shortcodes', 'alumni_chart') );
                add_shortcode( 'teams_list', array('MonLabo_shortcodes', 'teams_list') );
                add_shortcode( 'perso_panel', array('MonLabo_shortcodes', 'perso_panel') );
                add_shortcode( 'team_panel', array('MonLabo_shortcodes', 'team_panel') );

                //For development (not documented)
                add_shortcode( 'dev_team_name', array('MonLabo_shortcodes', 'dev_team_name') );
                add_shortcode( 'dev_team_logo', array('MonLabo_shortcodes', 'dev_team_logo') );

                //For retrocompatibility:
                add_shortcode( 'alumni', array('MonLabo_shortcodes', 'alumni') ); //For retrocompatibility - TO BE DELETED
                add_shortcode( 'perso_panel_automatic', array('MonLabo_shortcodes', 'perso_panel') );
                add_shortcode( 'members_list_automatic', array('MonLabo_shortcodes', 'members_list') );
                add_shortcode( 'perso_publications_automatic', array('MonLabo_shortcodes', 'publications_list') );
                add_shortcode( 'get_members_list', array('MonLabo_shortcodes', 'members_list') );

            }

            // Shortcodes for publications
            add_shortcode( 'publications_list', array('MonLabo_shortcodes_publications', 'publications_list') );
            add_shortcode( 'publications_automatic', array('MonLabo_shortcodes_publications', 'publications_list') ); //For retrocompatibility
            add_shortcode( "custom_publications_list", array('MonLabo_shortcodes_publications', 'custom_publications_list'));
            add_shortcode( "get_publications", array('MonLabo_shortcodes_publications', 'custom_publications_list') ); //For retrocompatibility

            //For translation
            load_plugin_textdomain( 'MonLabo', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );

            $options=get_option( 'MonLabo_settings_group2' );
            $color1=$options['MonLabo_foreground_color'];
            $color2=$options['MonLabo_address_color'];
            $address_font_size=$options['MonLabo_address_size'];
            $address_block_width=$options['MonLabo_address_block_width'];
            $perso_panel_width=$options['MonLabo_perso_panel_width'];
            $img_arrondi=$options['MonLabo_img_arrondi'];
            $name_size=$options['MonLabo_name_size'];

            wp_enqueue_style( 'MonLabo', plugin_dir_url( __FILE__ ).'MonLabo.css', array(), "2.7");
            $MonLabo_access_data = New MonLabo_access_data();
            //TODO: Retirer les commentaires et voir pourquoi cela plante
            /*$teams = $MonLabo_access_data->get_teams_information();
            $teams_id=array();
            $teams_color="";
            if (!empty($teams)){
              foreach ($teams as $team) {
                if (isset($team->color) and ($team->color!="")) {
                  $teams_color.=".MonLabo_persons_chart td .team_$team->id {color:'$team->color';}";
                }
              }
          }*/
            wp_add_inline_style( 'MonLabo', '
                .MonLabo_persons_list                     { color:'           .$color1.';}
                .MonLabo_persons_list a:visited           { color:'           .$color1.';}
                .MonLabo_persons_list ul li img           { border-color:'    .$color1.'; }
                .MonLabo_persons_list ul li a:hover img   { border-color:'    .$color1.'; }
                .MonLaboUser a, .MonLaboUser a:hover, .MonLaboUser a:visited  { color:'           .$color1.' !important;}
                a.MonLaboLink, a.MonLaboLink:hover, a.MonLaboLink:visited  { color:'           .$color1.' !important;}
                .publi_parisdescartes a  { color:'           .$color1.' !important;}
                .bandeau-personnel              { width:'           .$perso_panel_width.';}
                .bandeau-personnel .adresse     { color:'           .$color2.';             width:'.$address_block_width.';}
                .bandeau-personnel .adresse p   { font-size:'       .$address_font_size.';}
                .bandeau-personnel h1           { font-size:'       .$name_size.'; }
                .img-arrondi                    { border-radius:'   .$img_arrondi.'; }
                '
                /*.$teams_color*/
                .$options['MonLabo_custom_css']);

            //On ajoute un bouton éventuel sur la barre d'admin
            if (is_admin_bar_showing()) {
                require_once( dirname(__file__).'/admin/MonLabo_admin.php' );
                add_action('admin_bar_menu', 'MonLabo_admin_custom_toolbar_user_link', 999);
            }
        }
    }
}


/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-plugin-name-activator.php
 */
function activate_MonLabo() {

    $MonLabo_DB = New MonLabo_db();
    // * Pour test de debug migration 0.93 -> 1.0
    /*$options = get_option( 'MonLabo_settings_group0' );
    $options["MonLabo_activated_version"]="0.93";
    update_option('MonLabo_settings_group0', $options);*/
    $MonLabo_DB->migrate();
    $MonLabo_DB->create_tables();
    $MonLabo_DB->fill_tables_with_default_values_if_empty();

    //On vérifie que le numéro de version n'est pas actuel avant de mettre les valeurs par défaut
    $options = get_option( 'MonLabo_settings_group0' );
    if ((!isset($options))
        or (!isset($options["MonLabo_activated_version"]))){
        $MonLabo_DB->fill_options_with_default_values_if_empty();
    }

    //Mise à jour de la version activée
    $options = get_option( 'MonLabo_settings_group0' );
    //$MonLabo_menu_info = get_plugin_data( plugin_dir_path( __FILE__ ) .'../MonLabo.php' );
    $options["MonLabo_activated_version"]=get_version();
    update_option('MonLabo_settings_group0', $options);

}

//Deactivate the plugin
function deactivate_MonLabo()  { /*Do nothing*/ }

///////////////////////////////////////////////////////////////////////////////////////////
//                       MAIN CODE
///////////////////////////////////////////////////////////////////////////////////////////
if(class_exists('MonLabo'))
{
    // Installation hooks
    register_activation_hook    (__FILE__, 'activate_MonLabo');
    register_deactivation_hook  (__FILE__, 'deactivate_MonLabo');

    // instantiate the plugin class
    $laboPlugin = new MonLabo();

    // Add a link to the settings page onto the plugin page
    if(isset($laboPlugin)) {
        // Add the settings link to the plugins page
        function plugin_settings_link($links) {
            $settings_link = '<a href="'.admin_url('admin.php?page=MonLabo').'" title="' . __('Ouvre la page de configuration pour cette extension', 'MonLabo') . '">'.__('Configuration', 'MonLabo').'</a>';
            array_unshift($links, $settings_link);
            return $links;
        }

        $plugin = plugin_basename(__FILE__);
        add_filter("plugin_action_links_$plugin", 'plugin_settings_link');
    }
}
