<?php
if (!defined('WP_PLUGIN_DIR')) exit('No direct script access allowed');

///////////////////////////////////////////////////////////////////////////////////////////
//                       PLUGIN ADMIN CLASS DEFINITIONS
///////////////////////////////////////////////////////////////////////////////////////////
class MonLabo_admin {

    private $options;
    private $curent_section;
    private $curent_page;
    private $curent_settings_group;
    private $settings_section_parameters;

    function  __construct() {
        add_action('admin_init', array(&$this, 'admin_init'));
        add_action('admin_menu', array(&$this, 'add_menu'));

        $this->options['MonLabo_settings_group0']=get_option( 'MonLabo_settings_group0' );
        $this->options['MonLabo_settings_group1']=get_option( 'MonLabo_settings_group1' );
        $this->options['MonLabo_settings_group2']=get_option( 'MonLabo_settings_group2' );
        $this->options['MonLabo_settings_group3']=get_option( 'MonLabo_settings_group3' );
        $this->options['MonLabo_settings_group4']=get_option( 'MonLabo_settings_group4' );
        $this->options['MonLabo_settings_group5']=get_option( 'MonLabo_settings_group5' );

        add_action( 'admin_enqueue_scripts', array(&$this, 'MonLabo_enqueue_admin_scripts') );
        add_action( 'wp_ajax_update_member_thumbnail', array(&$this, 'update_member_thumbnail') );
    }

    function MonLabo_enqueue_admin_scripts( $hook_suffix ) {
        // first check that $hook_suffix is appropriate for your admin page
        wp_enqueue_style( 'wp-color-picker' );
        $MonLabo_menu_info = get_plugin_data( plugin_dir_path( __FILE__ ) .'../MonLabo.php' );
        wp_enqueue_script( 'MonLabo_admin-script', plugins_url( '/js/MonLabo_admin.js', __FILE__ ), array( 'wp-color-picker', 'jquery'), $MonLabo_menu_info['Version'], true );

        //Ne pas laisser d'espace vide quand un champs de formulaire est caché
        // => mettre les marges internes par défaut des conteneurs en question à 0.
        wp_add_inline_script("MonLabo_admin-script", "jQuery('input[type=hidden]').parents('td').css('padding', '0');"
                                                    ."jQuery('input[type=hidden]').parents('td').prev('th').css('padding', '0');"   );

	    // in JavaScript, object properties are accessed as ajax_object.ajax_url, ajax_object.we_value
	    wp_localize_script( 'MonLabo_admin-script', 'ajax_object_update_member_thumbnail', array( 'ajax_url' => admin_url( 'admin-ajax.php' )));
        wp_enqueue_media(); //On insère toutes les dépendances nécessaires pour l'affichage du menu média  //Obligatoire sous cette forme pour que le menu media fonctionne en AJAX
    }

    public function add_menu()
    {

        //Ajoute un menu accessible par les simples utilisateurs
        add_menu_page( 'Configuration des membres et des équipes du labo', 'Mon Labo', 'edit_pages', 'MonLabo', 'MonLabo_doc_render', plugins_url( 'images/logo16.gif', __FILE__ ), 999 );

        //Ajoute des sous-menu accessibles par les administrateurs
        if(current_user_can('manage_options'))
        {
            add_submenu_page( 'MonLabo', __('Configuration des membres et des équipes du labo', "MonLabo"), 'Documentation',        'manage_options', 'MonLabo');
            add_submenu_page( 'MonLabo',  __('Configurations générales', "MonLabo"),                         'Configuration',        'manage_options', 'MonLabo_config', 'MonLabo_admin_config_render');
            $options0=get_option( 'MonLabo_settings_group0' );
            if (isset($options0["MonLabo_uses_members_and_groups"]) && (intval($options0["MonLabo_uses_members_and_groups"])===1)) {
                add_submenu_page( 'MonLabo', __('Édite les personnels et les structures', "MonLabo"), __('Personnels et structures', "MonLabo"),   'manage_options', 'MonLabo_edit_members_and_groups', 'MonLabo_admin_edit_members_and_groups_render');
            }
        }
    }

    //----------------------------------------------------------------------------------
    // Nom:         error_message()
    // Description: Renvoie le code HTML pour afficher un message d'erreur.
    //
    // Parametres:
    //             $title   : Titre du cadre
    //             $message : Texte du cadre
    // Valeur de retour:
    //             Code HTML qu'il faudra afficher.
    //----------------------------------------------------------------------------------
    static public function error_message($title, $message) {
        $retval ='<div class="notice inline notice-error"><h3>'.$title.'</h3><p>'.$message.'</p></div>';
        return $retval;
    }

    //----------------------------------------------------------------------------------
    // Nom:         notice_message()
    // Description: Renvoie le code HTML pour afficher une boite d'alerte.
    //
    // Parametres:
    //             $type   : "error", "warning" ou "info"
    //             $title   : Titre du cadre
    //             $message : Texte du cadre
    // Valeur de retour:
    //             Code HTML qu'il faudra afficher.
    //----------------------------------------------------------------------------------
    static public function notice_message($type, $title, $message) {
        switch ($type) {
            case 'error':
                $noticeclass='notice-error';
                $dismissible='';
                break;

            case 'warning':
                $noticeclass='notice-warning';
                $dismissible='is-dismissible';
                break;

            default:
                $noticeclass='notice-info';
                $dismissible='is-dismissible';
                break;
        }
        $retval ='<div class="notice inline '.$noticeclass.' '.$dismissible.'"><h3>'.$title.'</h3><p>'.$message.'</p></div>';
        return $retval;
    }
    public function error_MonLabo_perso_page_parent() {
        echo MonLabo_admin::notice_message("error", __("Plugin MonLabo :","MonLabo"),
                                           __("Veuillez renseigner une page de rattachement des pages personnelles <a href=\"admin.php?page=MonLabo_config&tab=tab_appearance\">dans l'onglet 'aparence' des configurations générales du plugin</a>.","MonLabo"));
    }

    //hook into WP's admin_init action hook
    public function admin_init()
    {
        // Set up the settings for this plugin
        //Uses specific CSS for this admin
        $MonLabo_menu_info = get_plugin_data( plugin_dir_path( __FILE__ ) .'../MonLabo.php' );
        wp_enqueue_style( 'MonLabo_admin', plugin_dir_url( __FILE__ ).'css/MonLabo_admin.css', Array(), $MonLabo_menu_info['Version'] );

        // register the settings for this plugin
        register_setting( 'pluginSettings_group0', 'MonLabo_settings_group0' );
        register_setting( 'pluginSettings_group1', 'MonLabo_settings_group1' );
        register_setting( 'pluginSettings_group2', 'MonLabo_settings_group2' );
        register_setting( 'pluginSettings_group3', 'MonLabo_settings_group3' );
        register_setting( 'pluginSettings_group4', 'MonLabo_settings_group4' );
        register_setting( 'pluginSettings_group5', 'MonLabo_settings_group5' );

        $disableMembers=false;
        $disablePublications=false;

        // Onglet "Fonctionnalités"
        //-------------------------
        $this->curent_settings_group='MonLabo_settings_group0';
        $this->add_section('MonLabo_pluginPage_section_fonctionalites', __('Fonctions activables du plugin', "MonLabo"), 'MonLaboPageFunctionalities');
        $this->add_field('radio',   'MonLabo_publication_server_type',  __('Serveur de publication', "MonLabo"), false, " ");
        $this->add_field('checkbox2',    'MonLabo_uses_members_and_groups',         __('Personnels et équipes', "MonLabo"),         false, __("A décocher pour n'utiliser que l'extraction de publications Descartes Publi.", "MonLabo"));
        $options0=get_option( 'MonLabo_settings_group0' );
        if (!isset($options0["MonLabo_uses_members_and_groups"]) || ($options0["MonLabo_uses_members_and_groups"]!="1")) { $disableMembers=true; }
        if ($options0["MonLabo_publication_server_type"]!="aucun") { $disablePublications=true; }
        $this->add_field('checkbox2',    'MonLabo_uses_thematiques',     __('Groupes d\'équipes', "MonLabo"),         $disableMembers, __("Pour pouvoir grouper les équipes (thématique, groupe de recherche, axe...).", "MonLabo"));
        if (isset($options0["MonLabo_uses_thematiques"]) ) {
            $dissableThematic=$disableMembers;
        } else {
            $dissableThematic=true;
        }

        $this->add_field('text_field',  'MonLabo_teamGroupName_fr',    "", $dissableThematic, __("Nom du regroupement en français (singulier).", "MonLabo"));
        $this->add_field('text_field',  'MonLabo_teamGroupName_fr_pl', "", $dissableThematic, __("Nom du regroupement en français (pluriel).", "MonLabo"));
        $this->add_field('text_field',  'MonLabo_teamGroupName_en',    "", $dissableThematic, __("Nom du regroupement en anglais (singulier).", "MonLabo"));
        $this->add_field('text_field',  'MonLabo_teamGroupName_en_pl', "", $dissableThematic, __("Nom du regroupement en anglais (pluriel).", "MonLabo"));

        $this->add_field('checkbox2',    'MonLabo_uses_unites',          __('Unités multiples', "MonLabo"),                   $disableMembers, __("Pour une structure rassemblant plusieurs laboratoires.", "MonLabo"));
        $this->add_field('hidden',  'MonLabo_activated_version',       '', true);
        $this->add_section('MonLabo_pluginPage_section_fonctionalites2', __('Fonctions avancées', "MonLabo"), 'MonLaboPageFunctionalities');
        $this->add_field('checkbox2',    'MonLabo_uses_custom_texts_in_shortcodes',  __('Personnaliser les textes dans les shortcodes', "MonLabo"), $disableMembers, __("Affiche l'onglet \"Textes personnalisés\" pour pouvoir modifier les titres et les expressions qui sont affichés par défaut dans les paneaux, listes ou tables générés par le plugin.", "MonLabo"));
        $this->add_field('checkbox2',    'MonLabo_uses_custom_fields_for_staff',  __('Champs personnalisé pour les personnels', "MonLabo"), $disableMembers, __("Affiche l'onglet \"Champs personnalisés\" pour pouvoir ajouter des colonnes personnalisées aux tables d'utilisateurs.", "MonLabo"));

        // Onglet "Publications"
        //----------------------
        if ($options0["MonLabo_publication_server_type"]!="aucun") {
            $options4=get_option( 'MonLabo_settings_group4' );
            $this->curent_settings_group='MonLabo_settings_group4';

            switch ($options0["MonLabo_publication_server_type"]) {
                case 'hal':
                    $disableHal = false;
                    $disableDesccartesPubli = true;
                    $img="<a href='https://hal.archives-ouvertes.fr/'><img width='61' height='34' class='wp-image-8 alignleft wp-post-image' src='".plugins_url( 'images/hal.logo.png', __FILE__ )."' alt='logo HAL' /></a>";
                    break;
                case 'DescartesPubli':
                    $disableHal = true;
                    $disableDesccartesPubli = false;
                    $img="<img width='61' height='34' class='wp-image-8 alignleft wp-post-image' src='".plugins_url( 'images/DescartesPubli.logo.png', __FILE__ )."' alt='logo DescartesPubli'> ";
                    break;
                default:
                    $disableHal = true;
                    $disableDesccartesPubli = true;
                    $img="";
                    break;
            }

            $this->add_section('MonLabo_pluginPage_section_publication_servers', "&nbsp;".__('Lien avec le serveur de publications', "MonLabo").$img, 'MonLaboPageConfigPublications');
            $this->add_field('select',  'MonLabo_hal_publi_style', __('format d\'affichage des publications', "MonLabo"), $disableHal);
            $this->add_field('text_field',  'MonLabo_DescartesPubmed_api_url',       __('Adresse de l\'API publique de Descartes Publi', "MonLabo"), $disableDesccartesPubli);
            $this->add_field('select',  'MonLabo_DescartesPubmed_format', __('format d\'affichage des publications', "MonLabo"), $disableDesccartesPubli);
        }

        // Onglet "Apparence"
        //-------------------
        $this->curent_settings_group='MonLabo_settings_group2';
        $this->add_section('MonLabo_pluginPage_section_PagePersonnelles',     __('Pages personnelles', "MonLabo"), 'MonLaboPageAppearance');
        $this->add_field('select_page',    'MonLabo_perso_page_parent',     __('Page de rattachement des pages personnelles', "MonLabo"), $disableMembers);
        $this->add_section('MonLabo_pluginPage_section_liens',     __('Liens', "MonLabo"), 'MonLaboPageAppearance');
        $this->add_field('color_picker','MonLabo_foreground_color',     __('Couleur des liens et des contours photo', "MonLabo"),   $disableMembers);
        $this->add_section('MonLabo_pluginPage_section_photoPerso',     __('Photos personnelles', "MonLabo"), 'MonLaboPageAppearance');
        $this->add_field('text_field',  'MonLabo_img_arrondi',          __('Taux d\'arrondi des photos (0 à 50%)', "MonLabo"),     $disableMembers);
        $this->add_field('img_field',   'MonLabo_img_par_defaut',       __('Image par défaut', "MonLabo"),                         $disableMembers);
        $this->add_section('MonLabo_pluginPage_section_persoPanel', __('Bandeau personnel (perso panel)', "MonLabo"), 'MonLaboPageAppearance');
        $this->add_field('text_field',  'MonLabo_perso_panel_width',   __('Largeur du bandeau personnel', "MonLabo"),             $disableMembers);
        $this->add_field('text_field',  'MonLabo_name_size',            __('Taille du nom de la personne', "MonLabo"),             $disableMembers);
        $this->add_field('text_field',  'MonLabo_address_block_width',  __('Largeur du bloc d\'adresse', "MonLabo"),               $disableMembers);
        $this->add_field('color_picker','MonLabo_address_color',        __('Couleur de l\'adresse', "MonLabo"),                    $disableMembers);
        $this->add_field('text_field',  'MonLabo_address_size',         __('Taille du texte de l\'adresse', "MonLabo"),            $disableMembers);
        $this->add_section('MonLabo_pluginPage_section_csscustom', __('Mise en forme personnalisée', "MonLabo"), 'MonLaboPageAppearance');
        $this->add_field('text_area',   'MonLabo_custom_css',       __('Code CSS personnalisé', "MonLabo"),                  $disableMembers);

        // Onglet "Textes personalisés"
        //------------------------------
        if (isset($options0["MonLabo_uses_custom_texts_in_shortcodes"]) && (intval($options0["MonLabo_uses_custom_texts_in_shortcodes"])===1)) {
            $this->curent_settings_group='MonLabo_settings_group5';
            $this->add_section('MonLabo_pluginPage_section_custom_persons_texts',      __('Affichage des personnels', "MonLabo"), 'MonLaboPageCustomTexts');
            $this->add_field('text_field',  'MonLabo_custom_text_Personal_website_en',    __('Liens vers un site extérieur', "MonLabo").' (en/fr)');
            $this->add_field('text_field',  'MonLabo_custom_text_Personal_website_fr',    "");
            $this->add_field('text_field',  'MonLabo_custom_text_Team_leader_en',    __('Titre par défaut pour les chefs d\'équipe (en/fr/en pluriel/fr pluriel)', "MonLabo"));
            $this->add_field('text_field',  'MonLabo_custom_text_Team_leader_fr',    "");
            $this->add_field('text_field',  'MonLabo_custom_text_Team_leaders_en',    "");
            $this->add_field('text_field',  'MonLabo_custom_text_Team_leaders_fr',    "");
            $this->add_field('text_field',  "MonLabo_custom_text_Faculty_en", __('Catégories', "MonLabo")." (Faculty en/faculty fr/staff en/staff fr/postdocs en/postdocs en/students en/students fr/visitors en/visitors fr)");
            $this->add_field('text_field',  "MonLabo_custom_text_Faculty_fr", "");
            $this->add_field('text_field',  "MonLabo_custom_text_Staff_en" , "");
            $this->add_field('text_field',  "MonLabo_custom_text_Staff_fr", "");
            $this->add_field('text_field',  "MonLabo_custom_text_Postdocs_en", "");
            $this->add_field('text_field',  "MonLabo_custom_text_Postdocs_fr", "");
            $this->add_field('text_field',  "MonLabo_custom_text_Students_en", "");
            $this->add_field('text_field',  "MonLabo_custom_text_Students_fr" , "");
            $this->add_field('text_field',  "MonLabo_custom_text_Visitors_en", "");
            $this->add_field('text_field',  "MonLabo_custom_text_Visitors_fr", "");
            $this->add_section('MonLabo_pluginPage_section_custom_publications_texts', __('Affichage des publications', "MonLabo"), 'MonLaboPageCustomTexts');
            $this->add_field('text_field',  'MonLabo_custom_text_Recent_Publications_en', __('Titre par défaut devant la liste des publications', "MonLabo").' (en/fr)');
            $this->add_field('text_field',  'MonLabo_custom_text_Recent_Publications_fr', "");
        }


        // Onglet "Champs personnalisés"
        //------------------------------
        if (isset($options0["MonLabo_uses_custom_fields_for_staff"]) && (intval($options0["MonLabo_uses_custom_fields_for_staff"])===1)) {
            $this->curent_settings_group='MonLabo_settings_group3';
            $this->add_section('MonLabo_pluginPage_section_custom_fields',     __('Champs personnalisés pour la table des personnels', "MonLabo"), 'MonLaboPageCustomFields');
            $this->add_field('number', 'MonLabo_custom_fields_number', __('Nombre champs personnalisés (de 1 à 10)', "MonLabo"), $disableMembers);
            $options3=get_option( 'MonLabo_settings_group3' );
            if     (intval($options3["MonLabo_custom_fields_number"])<1) {  $nb=0; }
            elseif (intval($options3["MonLabo_custom_fields_number"])>10) {  $nb=10;}
            else   {  $nb=intval($options3["MonLabo_custom_fields_number"]); }
            for ($i=1; $i <= $nb; $i++) {
                $this->add_field('text_field', 'MonLabo_custom_field'.$i.'_title', sprintf(__('Titre du champs n°%u', "MonLabo"),$i), ("$nb">="$i" ? $disableMembers : true));
            }
        }

        // Met les valeurs par défaut aux options absentes de la base
        //-----------------------------------------------------------
        $MonLabo_DB = New MonLabo_db();
        $MonLabo_DB->fill_options_with_default_values_if_empty();

        //Alerte si MonLabo_perso_page_parent n'est pas configuré
        //-------------------------------------------------------
        $options2=get_option( 'MonLabo_settings_group2' );
        if($options2["MonLabo_perso_page_parent"]==="") {
            if (isset($options0["MonLabo_uses_members_and_groups"]) && ($options0["MonLabo_uses_members_and_groups"]=="1")) {
                add_action('admin_notices', array(&$this,'error_MonLabo_perso_page_parent'));
            } //Pas d'alerte si on désactive la fonctionnalité "Personnels et équipes"
        }

    }

    function add_section($section, $title, $page) {
        $this->current_section=$section;
        $this->current_page=$page;
        add_settings_section(   $section,                                                       // ID used to identify this section and with which to register options
                                __( $title, 'MonLabo' ),                                        // Title to be displayed on the administration page
                                array(&$this,'MonLabo_pluginPage_section_generic_callback'),    // Callback used to render the description of the section
                                $page);                                                         // Page on which to add this section of options
    }
    function MonLabo_pluginPage_section_generic_callback( $arg ) { /*echo __( '<h3>'.$arg['title'].'</h3>', 'MonLabo' ); */ }

    function  add_additional_field($type, $option_name, $disable=false, $description="") {
        $options_DEFAULT = get_options_DEFAULT();
        if($disable===true) {$type="hidden";}
        $title="";
        if($description==="") {
            if(!empty($options_DEFAULT[$this->curent_settings_group][$option_name])) {
                $description="Ex: <em>".$options_DEFAULT[$this->curent_settings_group][$option_name]."</em>";
            }
        }
        add_settings_field(
                            $option_name,
                            __( $title, 'MonLabo' ),
                            array(&$this, 'MonLabo_'.$type.'_generic_render'),
                            $this->current_page,
                            $this->current_section,
                            array('option_name'=>$option_name, 'description'=>$description, 'settings_group'=>$this->curent_settings_group, 'label_for' => "HSUlabel_for", 'class' => "HSUclass")
        );
    }

    function  add_field($type, $option_name, $title, $disable=false, $description="") {
        $options_DEFAULT = get_options_DEFAULT();
        if($disable===true) {$type="hidden";}
        if($type==="hidden") {$title="";}
        if($description==="") {
            if(!empty($options_DEFAULT[$this->curent_settings_group][$option_name])) {
                $description="Ex: <em>".$options_DEFAULT[$this->curent_settings_group][$option_name]."</em>";
            }
        }
        add_settings_field(
                            $option_name,
                            __( $title, 'MonLabo' ),
                            array(&$this, 'MonLabo_'.$type.'_generic_render'),
                            $this->current_page,
                            $this->current_section,
                            array('option_name'=>$option_name, 'description'=>$description, 'settings_group'=>$this->curent_settings_group)
        );
    }

    function MonLabo_select_generic_render( $args) {
        /*Refer function MonLabo_get_hal_publi_style_list {
            and function MonLabo_get_DescartesPubmed_format_list*/

        $function_name = str_replace("MonLabo_", "MonLabo_get_", $args['option_name'])."_list";
        $items = null;
        call_user_func_array($function_name, array(&$items));
        echo "<select name='".$args['settings_group']."[".$args['option_name']."]'>";
        foreach($items as $item) {
            $selected = (strcmp($this->options[$args['settings_group']][$args['option_name']], $item) == 0) ? 'selected="selected"' : '';
            echo "<option value='".$item."' ".$selected.">".$item."</option>";
        }
        echo "</select>";
    }

    function MonLabo_radio_generic_render( $args)  {
        $function_name = str_replace("MonLabo_", "MonLabo_get_", $args['option_name'])."_radio";
        $items = null;
        call_user_func_array($function_name, array(&$items));
        echo('<fieldset><legend>'.$args['description'].'</legend>');
        foreach($items as $itemKey=>$item) {
            $checked = (strcmp($this->options[$args['settings_group']][$args['option_name']], $itemKey) == 0) ? 'checked' : '';
            $id=uniqid();
            echo('<div>
                <input type="radio"
                       id="'.$id.'"
                       name="'.$args['settings_group']."[".$args['option_name']."]".'"
                       value="'.$itemKey.'"
                       '.$checked.' />
                <label for="'.$id.'">'.$item.'</label>
            </div>');

        }
        echo "</fieldset>";
    }

    function MonLabo_hidden_generic_render( $args) {
        echo("<input type='hidden' name='".$args['settings_group']."[".$args['option_name']."]' value='".$this->options[$args['settings_group']][$args['option_name']]."' />");
    }
    function MonLabo_text_field_generic_render( $args) {
        echo("<input type='text' name='".$args['settings_group']."[".$args['option_name']."]' value='".$this->options[$args['settings_group']][$args['option_name']]."' />");
        echo(" ".$args['description']);
    }
    function MonLabo_img_field_generic_render( $args) {
    	echo("	<div class='image-preview-wrapper'>");
    	echo("		<div class='MonLabo_persons_list'><ul><li><a href='#'><img width='60' height='60' id='image-preview' class='wp-image-8 alignleft img-arrondi wp-post-image' src='". $this->options[$args['settings_group']][$args['option_name']] ."' alt='". $this->options[$args['settings_group']][$args['option_name']] ."' /></a>");
        echo('</li></ul></div>');
	    echo("	</div>");
    	echo('	<input id="upload_image_button" type="button" class="button" value="'. __( 'Choisir ou téléverser', 'MonLabo' ).'"');
        echo(' onclick="image_media_menu(\''. __('Choisir une image', 'MonLabo').'\',\''. __('utiliser cette image', 'MonLabo').'\');" />');
        echo("	<input type='text' name='".$args['settings_group']."[".$args['option_name']."]' id='image_attachment_url' value='". $this->options[$args['settings_group']][$args['option_name']]."' />");
        echo("	<input type='hidden' name='not_used' id='image_attachment_id' value='0' />");
    }

    function MonLabo_color_picker_generic_render( $args) {
        echo("<input type='text' name='".$args['settings_group']."[".$args['option_name']."]' value='".$this->options[$args['settings_group']][$args['option_name']]."' class='my-color-field' data-default-color='#effeff' />");
        echo("<div style='float:left; height:30px;width:30px; background:".$this->options[$args['settings_group']][$args['option_name']].";border-radius:15px;box-shadow: 3px 3px 3px 0px #000;'></div><div style='float:left; margin:6px;'><small style='color:#777;'>".$this->options[$args['settings_group']][$args['option_name']]."</small></div>");
        echo(" ".$args['description']);
    }
    function MonLabo_text_area_generic_render( $args) {
        echo("<textarea name='".$args['settings_group']."[".$args['option_name']."]' rows='4' cols='50'>".$this->options[$args['settings_group']][$args['option_name']]."</textarea>");
        echo(" <pre>".$args['description']."</pre>");
    }
    function MonLabo_checkbox_generic_render( $args)  {
        // checked() is a wp function  https://codex.wordpress.org/Function_Reference/checked
        echo("<input type='checkbox' class='' name='".$args['settings_group']."[".$args['option_name']."]' ".checked( $this->options[$args['settings_group']][$args['option_name']], 1 , false)." value='1'>");
        echo(" ".$args['description']);
    }
    function MonLabo_checkbox2_generic_render( $args)  {
        echo('<div class="checkbox2">');
        $id=uniqid();
        // checked() is a wp function  https://codex.wordpress.org/Function_Reference/checked
        if (isset($this->options[$args['settings_group']][$args['option_name']])) {
            echo("<input type='checkbox' id='".$id."' name='".$args['settings_group']."[".$args['option_name']."]' ".checked( $this->options[$args['settings_group']][$args['option_name']], "1" , false)." value='1'>");
        } else {
            echo("<input type='checkbox' id='".$id."' name='".$args['settings_group']."[".$args['option_name']."]' value='1'>");
        }
        echo('<label for="'.$id.'"><span class="ui"></span></label>');
        echo('<p class="description_checkbox2">');
        echo($args['description']);
        echo('</p></div>');
    }
    function MonLabo_number_generic_render( $args)  {
        echo("<input type='number' name='".$args['settings_group']."[".$args['option_name']."]' value='".$this->options[$args['settings_group']][$args['option_name']]."' />");
        echo(" ".$args['description']);
    }

    function MonLabo_select_page_generic_render( $args)  {
        $retval='<br />';
        $retval.= wp_dropdown_pages(
        			array(
        				'name'              => $args['settings_group']."[".$args['option_name']."]",
        				'echo'              => 0,
        				'show_option_none'  => __( '&mdash; Select &mdash;' ),
        				'option_none_value' => '0',
        				'selected'          => $this->options[$args['settings_group']][$args['option_name']],
        			));

        echo(" ".$args['description']);
        echo $retval;
    }

    function cron() {
        //Si timer non défini attendre configuration adresse ou nombre utilisateur ou équipes > defaut
        //Quand timer défini; Regarder s'il n'a pas été executé depuis DELAY
        //  Si oui: Relancer fonction cron
    }

    //Ajax function to display the thumbnail image of member edition page
    function update_member_thumbnail() {
        require_once( dirname(__FILE__) . '/includes/inc-lib-forms.php' );
        $wp_post_id=getPOSTnumber('wp_post_id');

        // Nouveau membre
        //----------------
        if (is_null($wp_post_id)) {
            echo "<fieldset> <legend>".__("Choisir une image","MonLabo")." : </legend>";
            //Display default image
            echo '<a class="hover_zoom_square60"><img id="image-preview" src="'.$this->options['MonLabo_settings_group2']['MonLabo_img_par_defaut'].'" width="150" height="150" class="wp-image-6 alignright img-arrondi wp-post-image" alt="photo personne" /></a>';
            echo('<input id="upload_image_button" type="button" class="button" value="'. __( "Changer l'image par défaut", 'MonLabo' ).'"');
            echo(' onclick="image_media_menu(\''. __('Choisir une image', 'MonLabo').'\',\''. __('utiliser cette image', 'MonLabo').'\');" />');

        // Membre avec page valide
        //-----------------------
        } elseif ($wp_post_id>0) {
            echo "<fieldset> <legend>".__("Page wordpress choisie","MonLabo")." <small>(<a href='".get_edit_post_link( $wp_post_id )."'>".__("éditer la page", "MonLabo")."</a>)</small> : </legend>";
            echo '<a class="hover_zoom_square60">';
            if (has_post_thumbnail($wp_post_id)) {
                echo get_the_post_thumbnail($wp_post_id, array(150,150), array( 'class' => 'alignright img-arrondi',  'id'=>"image-preview"));
            } else {
                //Display default image
                echo '<img id="image-preview" src="'.$this->options['MonLabo_settings_group2']['MonLabo_img_par_defaut'].'" width="150" height="150" class="wp-image-6 alignright img-arrondi wp-post-image" alt="silhouette par défaut" />';
            }
            echo ('</a>');
            echo('<input id="upload_image_button" type="button" class="button" value="'. __( "Changer l'image de la page n°", 'MonLabo' ).$wp_post_id.'"');
            echo(' onclick="image_media_menu(\''. __('Choisir une image', 'MonLabo').'\',\''. __('utiliser cette image', 'MonLabo').'\');" />');

        // Membre avec page invalide
        //--------------------------
        } else {
            echo "<fieldset> <legend>".__("Choisissez une page Wordpress pour pouvoir avoir une image.","MonLabo")."</legend>";
        }
        echo("<input type='hidden' name='submit_image_attachment_url' id='image_attachment_url' value='__no_change__' />");
        echo("<input type='hidden' name='submit_image_attachment_id' id='image_attachment_id' value='__no_change__' />");
        echo "</fieldset>";

        wp_die();
    }
}

function MonLabo_admin_edit_members_and_groups_render() {
    //Interdit cette fonction si l'utilisateur n'a pas le droit "manage_options"
    if ( !current_user_can( 'manage_options' ) )  {
        wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
    }

    if( isset( $_GET[ 'tab' ] ) ) {  $active_tab = $_GET[ 'tab' ]; }
    else {                           $active_tab = 'tab_one';      }
    echo('<div class="wrap">');
    $options0=get_option( 'MonLabo_settings_group0' );
    echo MonLabo_get_title(esc_html( get_admin_page_title()));
    echo('<h2 class="nav-tab-wrapper">');
    echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_one" class="nav-tab ' . ($active_tab === 'tab_one' ? 'nav-tab-active' : '')  .'">'.__('Personnels', "MonLabo").'</a>');
    echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_two" class="nav-tab ' . ($active_tab === 'tab_two' ? 'nav-tab-active' : '')  .'">'.__('Équipes', "MonLabo").'</a>');
    if (isset($options0["MonLabo_uses_thematiques"]) && (intval($options0["MonLabo_uses_thematiques"])===1)) {
        echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_three" class="nav-tab ' . ($active_tab === 'tab_three' ? 'nav-tab-active' : '')  .'">'.MonLaboLib::get_teamGroupName_pl().'</a>');
    }
    if (isset($options0["MonLabo_uses_unites"]) && (intval($options0["MonLabo_uses_unites"])===1)) {
        echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_four" class="nav-tab ' . ($active_tab === 'tab_four' ? 'nav-tab-active' : '') .'">'.__('Unités', "MonLabo").'</a>');
    }
    echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_five" class="nav-tab ' . ($active_tab === 'tab_five' ? 'nav-tab-active' : '')  .'">'.__('Structure principale', "MonLabo").'</a>');
    echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_six" class="nav-tab ' . ($active_tab === 'tab_six' ? 'nav-tab-active' : '')  .'">'.__('Vue en tableaux', "MonLabo").'</a>');
    echo('<a href="?page=MonLabo_edit_members_and_groups&tab=tab_seven" class="nav-tab ' . ($active_tab === 'tab_seven' ? 'nav-tab-active' : '')  .'">'.__('Outils avancés', "MonLabo").'</a>');
    echo("</h2>");
    if( $active_tab === 'tab_one' )      { MonLabo_admin_edit_members_render(); }
    elseif( $active_tab === 'tab_two' )  { MonLabo_admin_edit_teams_render();   }
    elseif( $active_tab === 'tab_three') { MonLabo_admin_edit_thematics_render();  }
    elseif( $active_tab === 'tab_four')  { MonLabo_admin_edit_unites_render();  }
    elseif( $active_tab === 'tab_five')  { MonLabo_admin_edit_structure_principale_render();  }
    elseif( $active_tab === 'tab_six')  { MonLabo_admin_view_items_in_tables_render();  }
    elseif( $active_tab === 'tab_seven')  { MonLabo_admin_advanced_features_render();  }
    echo('</div>');
}

function MonLabo_get_hal_publi_style_list(&$list) {
    $list = array("apa", "hal", "ieee");
}
function MonLabo_get_DescartesPubmed_format_list(&$list) {
    $list = array("html_default"=>"html_default", "html_hal"=>"html_hal");
}

function MonLabo_get_publication_server_type_radio(&$list) {
    $list = array(  "hal"           =>"<a href='https://hal.archives-ouvertes.fr/'><img width='61' height='34' class='wp-image-8 alignleft wp-post-image' src='".plugins_url( 'images/hal.logo.png', __FILE__ )."' alt='logo HAL' /></a>&nbsp;HAL (API publique ouverte à tous)",
                    "DescartesPubli"=>"<img width='61' height='34' class='wp-image-8 alignleft wp-post-image' src='".plugins_url( 'images/DescartesPubli.logo.png', __FILE__ )."' alt='logo Descartes publi' />&nbsp;Descartes Publi (API réservée à l'Université ParisDescartes)",
                    "aucun" => "Aucun");
}

function MonLabo_get_title($soustitre=""){
    $retval="";
    //Verify version
    $MonLabo_menu_info = get_plugin_data( plugin_dir_path( __FILE__ ) .'../MonLabo.php' );
    $options0 = get_option( 'MonLabo_settings_group0' );
    if ($MonLabo_menu_info['Version']!=$options0['MonLabo_activated_version']) {
        echo(sprintf(__('<div class="error notice">
            <h3>Paramètres obsolètes</h3>
            <p>Le plugin a changé de paramètres et va se mettre à jour.<br />Merci de recharger la page.</p>
            <p></p>
            <p><strong>Si le message persiste, merci de désactiver et réactiver le plugin.</strong></p>
            <p>(Ce plugin à été mis à jour à la version %s depuis  la version %s)</p>
            </div>', "MonLabo"),$MonLabo_menu_info['Version'], $options0['MonLabo_activated_version']));
    }
    //Display title
    $retval.="<h1>";
    $retval.= __($MonLabo_menu_info['Name'], "MonLabo")." <small>(".sprintf(__('version %s', "MonLabo"), $MonLabo_menu_info['Version']).")</small> ". '<a href="'.$MonLabo_menu_info['PluginURI'].'" class="MonLabo_uri">'. str_replace(array("http://", "https://"), "", $MonLabo_menu_info['PluginURI'])."</a>";
    $retval.="</h1>";
    if ($soustitre!="") {
        $retval.=" <h2>".$soustitre."</h2>";
    }

    //Permit to migrate if necessary
    activate_MonLabo();
    return $retval;
}

///////////////////////////////////////////////////////////////////////////////////////////
//                                  CONFIGURATION MENU
///////////////////////////////////////////////////////////////////////////////////////////
function MonLabo_doc_render() {
    $MonLabo_menu_info = get_plugin_data( plugin_dir_path( __FILE__ ) .'../MonLabo.php' );
    echo('<div class="MonLabo_logo_tutelle"><a href="http://www.u-pec.fr/"><img src="'.plugins_url( 'images/logo-upec.png', __FILE__ ).'" alt="Logo U-PEC" /></a></div>');
    echo('<div class="MonLabo_logo_tutelle"><a href="http://www.parisdescartes.fr/"><img src="'.plugins_url( 'images/logo-upd.png', __FILE__ ).'"  alt="Logo ParisDescartes" /></a></div>');
    echo('<div class="MonLabo_logo_tutelle"><a href="http://www.cnrs.fr/"><img src="'.plugins_url( 'images/logo-cnrs.png', __FILE__ ).'" alt="Logo CNRS" /></a></div>');
    echo MonLabo_get_title();
    echo("<h2>".__($MonLabo_menu_info['Description'], "MonLabo")."</h2>");
    require_once( dirname(__FILE__).'/MonLabo_doc.php' );
    MonLabo_help_render();
}

function MonLabo_admin_config_render() {
    //Interdit cette fonction si l'utilisateur n'a pas le droit "manage_options"
    if ( !current_user_can( 'manage_options' ) )  {
        wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
    }

    if( isset( $_GET[ 'tab' ] ) ) {  $active_tab = $_GET[ 'tab' ]; }
    else {                           $active_tab = 'tab_functionalities';      }
    echo('<div class="wrap">');
    $options0=get_option( 'MonLabo_settings_group0' );
    echo MonLabo_get_title(esc_html( __(get_admin_page_title(), "MonLabo")));
    settings_errors();
    echo('<form method="post" action="options.php">');

    echo('<h2 class="nav-tab-wrapper">');
    echo('<a href="?page=MonLabo_config&tab=tab_functionalities" class="nav-tab '. ($active_tab === 'tab_functionalities' ? 'nav-tab-active' : '').'">'.__('Fonctionnalités', "MonLabo").'</a>');
    if ($options0["MonLabo_publication_server_type"]!="aucun") {
        echo('<a href="?page=MonLabo_config&tab=tab_configPublications" class="nav-tab '. ($active_tab === 'tab_configPublications' ? 'nav-tab-active' : '').'">'.__('Publications', "MonLabo").'</a>');
    }
    if (isset($options0['MonLabo_uses_members_and_groups']) && (intval($options0["MonLabo_uses_members_and_groups"])===1 )) {
        //echo('<a href="?page=MonLabo_config&tab=tab_one" class="nav-tab '. ($active_tab === 'tab_one' ? 'nav-tab-active' : '').'">'.__('Coordonnées', "MonLabo").'</a>');
        echo('<a href="?page=MonLabo_config&tab=tab_appearance" class="nav-tab '. ($active_tab === 'tab_appearance' ? 'nav-tab-active' : '').'">'.__('Apparence', "MonLabo").'</a>');
        if (isset($options0["MonLabo_uses_custom_fields_for_staff"]) && (intval($options0["MonLabo_uses_custom_fields_for_staff"])===1)) {
            echo('<a href="?page=MonLabo_config&tab=tab_customFields" class="nav-tab '. ($active_tab === 'tab_customFields' ? 'nav-tab-active' : '').'">'.__('Champs personnalisés', "MonLabo").'</a>');
        }
    }
    if (isset($options0["MonLabo_uses_custom_texts_in_shortcodes"]) && (intval($options0["MonLabo_uses_custom_texts_in_shortcodes"])===1)) {
        echo('<a href="?page=MonLabo_config&tab=tab_customTexts" class="nav-tab '. ($active_tab === 'tab_customTexts' ? 'nav-tab-active' : '').'">'.__('Textes personnalisés', "MonLabo").'</a>');
    }
    echo("</h2>");

    if( $active_tab === 'tab_functionalities' )      {
        settings_fields( 'pluginSettings_group0' );
        do_settings_sections( 'MonLaboPageFunctionalities' );
    }
    if( $active_tab === 'tab_configPublications' )      {
         settings_fields( 'pluginSettings_group4' );
         do_settings_sections( 'MonLaboPageConfigPublications' );
    }
    /*if( $active_tab === 'tab_one' )      {
        settings_fields( 'pluginSettings_group1' );
        do_settings_sections( 'MonLaboPage1' );
    }*/
    elseif( $active_tab === 'tab_appearance' )  {
        settings_fields( 'pluginSettings_group2' );
        do_settings_sections( 'MonLaboPageAppearance' );
    }
    elseif( $active_tab === 'tab_customTexts' )  {
        settings_fields( 'pluginSettings_group5' );
        do_settings_sections( 'MonLaboPageCustomTexts' );
    }
    elseif( $active_tab === 'tab_customFields' )  {
        settings_fields( 'pluginSettings_group3' );
        do_settings_sections( 'MonLaboPageCustomFields' );
    }
    submit_button();
    echo("</form>");
    echo('</div>');
}

function MonLabo_admin_edit_members_render() {
    require_once( dirname(__FILE__) . '/MonLabo_edit_members.php' );
    echo('<div class="wrap">');
    echo edit_members_form();
    echo('</div>');
}
function MonLabo_admin_edit_teams_render() {
    require_once( dirname(__FILE__) . '/MonLabo_edit_members.php' );
    echo('<div class="wrap">');
    echo edit_teams_form();
    echo('</div>');
}
function MonLabo_admin_edit_unites_render() {
    require_once( dirname(__FILE__) . '/MonLabo_edit_members.php' );
    echo("<h3>".__("Si votre structure n'est qu'un seul laboratoire, ne pas remplir.","MonLabo")."</h3>");
    echo('<div class="wrap">');
    echo edit_unites_form();
    echo('</div>');
}

function MonLabo_admin_edit_structure_principale_render() {
    require_once( dirname(__FILE__) . '/MonLabo_edit_members.php' );
    echo('<div class="wrap">');
    echo edit_structure_principale_form();
    echo('</div>');
}

function MonLabo_admin_edit_thematics_render() {
    require_once( dirname(__FILE__) . '/MonLabo_edit_members.php' );
    echo('<div class="wrap">');
    echo edit_thematics_form();
    echo('</div>');
}

function MonLabo_admin_view_items_in_tables_render() {
    require_once( dirname(__FILE__) . '/includes/inc-lib-tables.php' );

    $options0=get_option( 'MonLabo_settings_group0' );
    echo('<div class="wrap admin_tables" id="haut">');
    echo(__('D\'un coup d\'œil, visualisez toute votre base.', "MonLabo").' ');
    echo(__('Raccourcis :', "MonLabo").' <a href="#personnes_table">'.__('Personnes', "MonLabo").'</a>, <a href="#equipes_table">'.__('Équipes', "MonLabo").'</a>');
    if (isset($options0["MonLabo_uses_thematiques"]) && (intval($options0["MonLabo_uses_thematiques"])===1)) {
        echo(', <a href="#thematiques_table">'.__('Thématiques', "MonLabo").'</a>');
    }
    if (isset($options0["MonLabo_uses_unites"]) && (intval($options0["MonLabo_uses_unites"])===1)) {
        echo(', <a href="#unites_table">'.__('Unités', "MonLabo").'</a>');
    }

    echo('<h3 id="personnes_table">'.__('Personnes', "MonLabo").'</h3>');
    echo("<h4>"._x('Actifs', "nom_pluriel", "MonLabo")."</h4>");
    echo(generate_table_admin_for_persons($status="actif"));
    echo("<h4>"._x('Alumni', "nom_pluriel", "MonLabo")."</h4>");
    echo(generate_table_admin_for_persons($status="alumni"));
    echo('<a href="#haut">'.__('Retour en haut de la page', "MonLabo").'</a>');

    echo('<h3 id="equipes_table">'.__('Équipes', "MonLabo").'</h3>');
    echo generate_table_admin_for_teams();
    echo('<a href="#haut">'.__('Retour en haut de la page', "MonLabo").'</a>');

    if (isset($options0["MonLabo_uses_thematiques"]) && (intval($options0["MonLabo_uses_thematiques"])===1)) {
        echo('<h3 id="thematiques_table">'.__('Thématiques', "MonLabo").'</h3>');
        echo generate_table_admin_for_thematics();
        echo('<a href="#haut">'.__('Retour en haut de la page', "MonLabo").'</a>');
    }

    if (isset($options0["MonLabo_uses_unites"]) && (intval($options0["MonLabo_uses_unites"])===1)) {
        echo('<h3 id="unites_table">'.__('Unités', "MonLabo").'</h3>');
        echo generate_table_admin_for_units();
        echo('<a href="#haut">'.__('Retour en haut de la page', "MonLabo").'</a>');
    }
    echo('</div>');
}


function MonLabo_admin_advanced_features_render() {
    require_once( dirname(__FILE__) . '/MonLabo_edit_members.php' );
    echo('<div class="wrap">');
    echo display_advanced_features_for_mmebers();
    echo('</div>');
}

// add a link to the WP Toolbar
function MonLabo_admin_custom_toolbar_user_link($wp_admin_bar) {
    $MonLabo_access_data = New MonLabo_access_data();
    $wp_post_id=get_the_ID();
    if(!empty($wp_post_id)) {
        $iconurl = plugin_dir_url(__FILE__ )."images/logo16.gif";
        $iconspan = '<span class="custom-icon" style="color: #72777c; float:left; width:16px !important; height:16px !important; margin: 7px 5px 0px 5px !important; background-image:url(\''.$iconurl.'\');"></span>';

        $id=$MonLabo_access_data->get_person_id_from_wpPostId($wp_post_id);
        if(!empty($id)) {
            $title = __('Édite cette personne', 'MonLabo');
            $tab = 'tab_one';
        } else {
            $id=$MonLabo_access_data->get_team_id_from_wpPostId($wp_post_id);
            if(!empty($id)) {
                $title = __('Édite cette équipe', 'MonLabo');
                $tab = 'tab_two';
            } else {
                $id=$MonLabo_access_data->get_unit_id_from_wpPostId($wp_post_id);
                $title = __('Édite cette unité', 'MonLabo');
                $tab = 'tab_four';
            }
        }
        if(!empty($id)) {
            $args = array(
                'id' => 'MonLabo_edit_link',
                'title' => $iconspan.$title,
                'href' => get_admin_url().'admin.php?page=MonLabo_edit_members_and_groups&tab='.$tab.'&submit_item='.$id,
            );
            $wp_admin_bar->add_node($args);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////////////////
//                       MAIN CODE
///////////////////////////////////////////////////////////////////////////////////////////
if(class_exists('MonLabo'))
{
    if(class_exists('MonLabo_admin'))
    {
        // instantiate the plugin admin class
        $laboAdminPlugin = new MonLabo_admin();
    }

    //Replace admin menu by simple help page for non-admin users
    // Add Toolbar Menus
    /*function custom_toolbar() {
        if(!current_user_can('manage_options')) {
            global $wp_admin_bar;

            $args = array(  'id'     => 'MonLaboToolbar_id',
                            'title'  => __( '<img src="'.plugins_url( 'images/logo16.gif', __FILE__ ).'"/> Mon Labo', 'text_domain' ),
                            'href'   => plugins_url( 'MonLabo_user_doc.php', __FILE__ ),
                            );
            $wp_admin_bar->add_menu( $args );
        }
    }
    add_action( 'wp_before_admin_bar_render', 'custom_toolbar', 999 );*/
}

?>
