<?php
/*
class MonLabo_access_data {

    // ACCESSOR METHODS
    -------------------------------------------------------------
    // Get persons methods
    get_person_information ($person_id)
    get_person_information_from_wpPostId ($wp_post_id)
    get_persons_information ($status="actif")
    get_persons_information_for_a_team ($team_id, $status="actif")
    get_persons_information_for_an_unit ($unit_id, $status="actif")
    get_persons_information_for_a_category ($category, $status="actif")
    get_persons_information_for_a_category_in_an_unit ($category, $unit, $status="actif")
    get_persons_information_for_a_category_in_a_team ($category, $team, $status="actif")
    get_persons_name_for_a_team ($team_id, $status="actif")
    get_persons_id_for_a_team ($team_id, $status="actif")
    get_person_name ($person_id)
    get_persons_name ($status="actif")
    get_person_id_from_wpPostId ($wp_post_id)

    // Get persons with special functions methods
    get_mentors_name_for_a_person ($person_id, $status="actif")
    get_mentors_id_for_a_person ($person_id, $status="actif")
    get_mentors_information_for_a_person ($team, $status="actif")
    get_students_name_for_a_person ($person_id, $status="actif")
    get_students_id_for_a_person ($person_id, $status="actif")
    get_students_information_for_a_person ($team, $status="actif")
    get_leaders_name_for_a_team ($team_id, $status="actif")
    get_leaders_id_for_a_team ($team_id, $status="actif")
    get_leaders_information_for_a_team ($team, $status="actif")
    get_non_leaders_information_for_a_team ($team, $status="actif")
    get_directors_name ($status="actif")
    get_directors_information ($status="actif")
    get_directors_name_for_an_unit ($unit_id, $status="actif")
    get_directors_id_for_an_unit ($unit_id, $status="actif")
    get_units_id_for_a_director ($person_id, $status="actif")

    // Get teams methods
    get_team_information ($team_id)
    get_teams_information ()
    get_teams_information_for_an_unit ($unit_id)
    get_teams_id_for_an_unit ($unit_id)
    get_teams_information_for_a_person ($person_id)
    get_team_name ($team_id, $lang)
    get_teams_name ($lang)
    get_teams_name_for_a_person ($person_id, $lang)
    get_teams_id_for_a_person ($person_id)
    get_team_id_from_wpPostId ($wp_post_id)
    get_teams_id_for_a_thematic ($thematic_id)
    get_teams_id_for_a_leader($person_id, $status="actif")

    // Get thematics methods
    get_thematic_information ($thematic_id)
    get_thematics_information ()
    get_thematic_name ($thematic_id, $lang)
    get_thematics_name ($lang)
    get_thematics_name_for_a_team ($team_id, $lang)
    get_thematics_id_for_a_team ($team_id)

    // Get units methods
    get_unit_information ($unit_id)
    get_unit_id_from_wpPostId ($wp_post_id)
    get_units_name ($lang)

    // other methods
    get_multilingual_functions_by_category ()

    // MUTATOR METHODS
      -------------------------------------------------------------
    * Insert Methods
    insert_person ($data)
    insert_team ($data)
    insert_thematic ($data)
    insert_unit ($data)
    add_mentor_to_a_person ($mentor_id, $person_id)
    add_person_to_a_team ($person_id, $team_id)
    add_leader_to_a_team ($leader_id, $team_id)
    add_thematic_to_a_team ($thematic_id, $team_id)
    add_director_to_an_unit ($director_id, $unit_id)
    remove_director_from_an_unit ($director_id, $unit_id)

    * Update Methods
    update_person ($person_id, $data)
    update_team ($team_id, $data)
    update_unit ($unit_id, $data)
    update_thematic ($thematic_id, $data)
    private set_teams_for_a_person ($new_teams, $person_id)
    private set_mentors_for_a_person ($new_mentors, $person_id)
    private set_students_for_a_person ($new_mentors, $person_id)
    private set_leaders_for_a_team ($new_leaders, $team_id)
    private set_thematics_for_a_team ($new_thematics, $team_id)
    private set_directors_for_an_unit ($new_directors, $unit_id)

    * Delete Methods
    delete_person ($person_id)
    delete_team ($team_id)
    delete_thematic ($thematic_id)
    delete_unit ($unit_id)
    private delete_relations_between_teams_and_a_person ($person_id)
    private delete_relations_between_a_person_and_his_or_her_mentors ($person_id)
    private delete_relations_between_a_person_and_his_or_her_students ($person_id)
    private delete_relations_between_persons_and_a_team ($team_id)
    private delete_relations_between_thematics_and_a_team ($team_id)
    private delete_relations_between_leaders_and_a_team ($team_id)
    private delete_relations_between_teams_and_a_thematic ($thematic_id)
    private delete_relations_between_directors_and_a_unit ($unit_id)
    private delete_relations_between_units_and_a_director($person_id)
}
*/


require_once( dirname(__FILE__) . '/constants.php' );

class MonLabo_access_data {

    private $wpdb;

    public function __construct() {
        global $wpdb;
        $this->wpdb = $wpdb;
    }

    ////////////////////////////////////////////////////////////////////
    //  PRIVATE METHODS
    ////////////////////////////////////////////////////////////////////
    /**
     * Transform input into clean array
     * @param $input array or coma separated items in a string
     * @return array|null always in array without empty entries
     * Array() if no result is found
     * @access private
     */
    private function input_array_of_ids($inputs) {
        if (!is_array($inputs)) {
            $inputs=str_replace("|", ",", $inputs); //Pouvoir utiliser | comme separateur
            $inputs=explode(",",$inputs); //Comma separated into array
        }
        //Suppress empty entries in array
        if(!empty($inputs)) {
            foreach($inputs as $key=>$input) {
                $input=intval(trim($input));
                if (empty($input) or ($input<1)) {
                    unset($inputs[$key]);
                } else {
                    $inputs[$key]=$input;
                }
            }
        }
        return $inputs;
    }


    /**
     * Trigger an error if parameter is not numeric
     * @param $parameter to test
     * @return true if error triggered, else false
     * @access private
     */
    private function trigger_error_if_non_numeric($parameter){
        if (!is_numeric($parameter)) {
            trigger_error("The function can not be executed because of an invalid parameter.", E_USER_WARNING);
            return true; //Si l'ID n'est pas un nombre on évite de lancer une requette invalide
        }
        return false;
    }

    /**
     * Trigger an error if parameter is not set
     * @param $parameter to test
     * @return true if error triggered, else false
     * @access private
     */
    private function trigger_error_if_not_set($parameter){
        if (!isset($parameter)) {
            trigger_error("The function can not be executed because of an invalid parameter.", E_USER_WARNING);
            return true; //Si l'ID n'est pas un nombre on évite de lancer une requette invalide
        }
        return false;
    }

    /**
     * Trigger an error if parameter is empty or not an array
     * @param $parameter to test
     * @return true if error triggered, else false
     * @access private
     */
    private function trigger_error_if_empty_or_not_array($parameter){
        if (!is_array($parameter) or empty($parameter)) {
            trigger_error("The function can not be executed because of an invalid parameter.", E_USER_WARNING);
            return true; //Si l'ID n'est pas un nombre on évite de lancer une requette invalide
        }
        return false;
    }


    /**
     *
     * @param $object_table
     * @return mixed|null array($id => $id)
     * NULL if no result is found
     * @access private
     */
    private function tidy_object_table_and_leave_only_one_column($object_table, $index_attribute_name="id", $value_attribute_name=NULL) {
        $id_table = array();
        if($value_attribute_name===NULL) { $value_attribute_name = $index_attribute_name; }
        if(!empty($object_table)) {
            foreach ($object_table as $one_object) {
                if(property_exists($one_object, $index_attribute_name)) { //In theory always enter in
                    $id_table[$one_object->$index_attribute_name] = $one_object->$value_attribute_name;
                }
            }
        }
        return (empty($id_table) ? NULL : $id_table);
    }


    /**
     * Index an object table by an attribute
     * @param $object_table
     * @return mixed|null array($id => $id)
     * NULL if no result is found
     * @access private
     */
    private function tidy_object_table($object_table, $attribute_name="id") {
        $id_table = array();
        if(!empty($object_table)) {
            foreach ($object_table as $one_object) {
                if(property_exists($one_object, $attribute_name)) { //In theory always enter in
                    $id_table[$one_object->$attribute_name] = $one_object;
                }
            }
        }
        return (empty($id_table) ? NULL : $id_table);
    }

    /**
     * Index an object table by an attribute extract name
     * @param $object_table
     * @return mixed|null array($id => $id)
     * NULL if no result is found
     * @access private
     */
    private function tidy_object_table_for_name($object_table, $attribute_name="id") {
        $id_table = array();
        if(!empty($object_table)) {
            foreach ($object_table as $one_object) {
                if(property_exists($one_object, $attribute_name)) { //In theory always enter in
                    $id_table[$one_object->$attribute_name] = $one_object->first_name . " ". mb_strtoupper($one_object->last_name, 'UTF-8');
                }
            }
        }
        return (empty($id_table) ? NULL : $id_table);
    }

    /**
     * Return the column name depending the language
     * @param $lang language (en or local language)
     * @return "name_<lang>"
     * @access private
     */
    private function column_name_for_language($lang) {
        switch ($lang) {
            case "en-GB" :
            case "en-US" :
            case "en" :
                return 'name_en';
            default : // fr
                return 'name_fr';
        }
    }

    ////////////////////////////////////////////////////////////////////
    // ACCESSOR METHODS
    ////////////////////////////////////////////////////////////////////

    /*******************************************************************
     * Generic get Methods
    *******************************************************************/

    /**
     * Get a row of a table
     * @param string $table_name name of the table
     * @param int $table_id id of the row
     * @return array|null array(information_name => information_value)
     * NULL if no result is found
     * @access private
     */
    private function get_table_row($table_name, $table_id) {
        if($this->trigger_error_if_non_numeric($table_id)) {return NULL;}
        return ($this->get_table_row_from_conditions($table_name, array("id" => $table_id)));
    }

    /**
     * Get a row of a table
     * @param string $table_name name of the table
     * @param array $columns_conditions condition array(column_name => column_value)
     * @return array|null array(information_name => information_value)
     * NULL if no result is found
     * @access private
     */
    private function get_table_row_from_conditions($table_name, $columns_conditions) {
        if($this->trigger_error_if_empty_or_not_array($columns_conditions)) {return NULL;}
        $condition_array = array();
        if (isset($columns_conditions)) {
            foreach ($columns_conditions as $key => $value)
                $condition_array[] = " u.".$key." = \"".$value."\"";
        }
        $condition = " WHERE".implode(" AND",$condition_array);
        $sql = "SELECT * FROM ".$table_name." AS u".$condition;
        $retval = $this->wpdb->get_row($sql);
        return (empty($retval) ? NULL : $retval);
    }


    /**
     * Get one variable of a table
     * @param string $table_name name of the table
     * @param int $table_id id of the row
     * @param int $table_column column name of the variable
     * @return string|null single variable of the database
     * NULL if no result is found
     * @access private
     */
    private function get_table_variable_from_id($table_name, $table_id, $table_column) {
        if (is_array($table_column)) {
            $table_id = "invalid"; //trigger error next line
        }
        if($this->trigger_error_if_non_numeric($table_id)) {return NULL;}
        $sql = $this->wpdb->prepare("SELECT ".$table_column." FROM ".$table_name." WHERE id = \"%d\"", $table_id);
        $retval = $this->wpdb->get_var($sql);
        return (empty($retval) ? NULL : $retval);
    }

    /**
     * Get variables of a table
     * @param string $table_name name of the table
     * @param mixed int|array $table_columns column name of the variable or an array of column names
     * @param array $columns_conditions condition array(column_name => column_value)
     * @return array |null array(information_name => information_value)
     * NULL if no result is found
     * @access private
     */
    private function get_table_variables_from_column($table_name, $table_columns, $columns_conditions, $returntype = OBJECT_K) {
        //WARNING: OBJECT_K has to be only used for single keys tables. On multi keys tables, duplicates single columns will be discarded.
        $condition_array = array();
        if (isset($columns_conditions)) {
            foreach ($columns_conditions as $key => $value)
                $condition_array[] = " u.".$key." = \"".$value."\"";
        }
        $condition = " WHERE".implode(" AND",$condition_array);
        if (is_array($table_columns)) {
            $columns=implode(', ', $table_columns);
        } else {
            $columns=$table_columns;
        }
        $sql = "SELECT ".$columns." FROM ".$table_name." AS u" . $condition;
        $retval = $this->wpdb->get_results($sql, $returntype);
        return (empty($retval) ? NULL : $retval);
    }


    /**
     * Get variables of a row of a table
     * @param string $table_name name of the table
     * @param int $table_id id of the row
     * @param array $table_columns list of column name of the variable
     * @return string|null single variable of the database
     * NULL if no result is found
     * @access private
     */
    private function get_table_variables_from_id($table_name, $table_id, $table_columns) {
        if($this->trigger_error_if_non_numeric($table_id)) {return NULL;}
        if (is_array($table_columns)) {
            $columns=implode(', ', $table_columns);
        } else {
            $columns=$table_columns;
        }
        $sql = $this->wpdb->prepare("SELECT ".$columns." FROM ".$table_name." WHERE id = \"%d\"", $table_id);
        $retval = $this->wpdb->get_row($sql);
        return (empty($retval) ? NULL : $retval);
    }

    /**
     * Get all values of a columns of a table
     * @param string $table_name name of the table
     * @param array $table_columns list of column name of the variable
     * @param array $order_by (facultatif) list of column name for ascendant order
     * @return string|null single variable of the database
     * NULL if no result is found
     * @access private
     */
    private function get_table_column($table_name, $table_columns, $order_by = array(), $column_condition = array(), $returntype = OBJECT_K) {
        //WARNING: OBJECT_K has to be only used for single keys tables. On multi keys tables, duplicates single columns will be discarded.
        $columns = implode(", ", $table_columns);
        $order = "";
        if (!empty($order_by)) {
            $order = " ORDER BY ".implode(" ASC, ", $order_by)." ASC";
        }
        $condition = "";
        if (!empty($column_condition)) {
            $condition = " WHERE ";
            $conditions=array();
            foreach ($column_condition as $key => $value) {
                $conditions[] = $key." = \"".$value."\" ";
            }
            $condition .= implode(" AND ", $conditions);
        }
        $sql = "SELECT ".$columns." FROM ".$table_name.$condition.$order;
        $retval = $this->wpdb->get_results($sql, $returntype);
        return (empty($retval) ? NULL : $retval);
    }

    /**
     * Get all values of a columns of a row of a step table
     *
     * @param string $union_table_name name of the union table
     * @param string $step_table_name name of the step table
     * @param array $union_table_columns list of column name of the union table array (principal_table, step_table)
     * @param array $step_table_columns list of column name of the step table
     * @param int $principal_table_id id of the row of the principal table
     * @param array $order_by (facultatif) list of column name for ascendant order
     * @param array $add_union_condition (facultatif) condition array(column_name => column_value) for union table
     * @param array $add_step_condition (facultatif) condition array(column_name => column_value) for step table
     * @return string|null single variable of the database
     * NULL if no result is found
     * @access private
     */
    private function get_step_table_columns($union_table_name, $step_table_name, $union_table_columns, $step_table_columns,
        $principal_table_id, $order_by = array(), $add_union_condition = array(), $add_step_condition = array()) {
        $columns = "s.".$step_table_columns[0];
        for ($i = 1; $i < count($step_table_columns); $i++) {
            $columns .= ", s.".$step_table_columns[$i];
        }
        $order = "";
        if (isset($order_by)) {
            $order = " ORDER BY ".implode(" ASC, ", $order_by)." ASC";
        }
        $union_condition = "";
        if (isset($add_union_condition)) {
        foreach ($add_union_condition as $key => $value)
            $union_condition .= " AND u.".$key." = \"".$value."\" ";
        }
        $step_condition = "";
        if (isset($add_step_condition)) {
        foreach ($add_step_condition as $key => $value)
            $step_condition .= " AND s.".$key." = \"".$value."\" ";
        }
        $sql = $this->wpdb->prepare("SELECT ".$columns." FROM ".$step_table_name." AS s, ".$union_table_name." AS u WHERE u.".$union_table_columns[0]." = \"%d\" AND u.".$union_table_columns[1]." = s.id ".$union_condition.$step_condition.$order, $principal_table_id);
        $retval = $this->wpdb->get_results($sql, OBJECT);
        return (empty($retval) ? NULL : $retval);
    }

    /**
     * Get all values of a columns of a row of a step table
     * @param string $union_table_name name of the union table
     * @param string $step_table_name name of the step table
     * @param array $union_table_columns list of column name of the union table array (principal_table, step_table)
     * @param array $step_table_columns list of column name of the step table
     * @param array $order_by (facultatif) list of column name for ascendant order
     * @param array $add_union_condition (facultatif) condition array(column_name => column_value) for union table
     * @param array $add_step_condition (facultatif) condition array(column_name => column_value) for step table
     * @return string|null single variable of the database
     * NULL if no result is found
     * @access private
     */
    private function get_all_step_table_columns($union_table_name, $step_table_name, $union_table_columns, $step_table_columns, $order_by = array(), $add_union_condition = array(), $add_step_condition = array()) {
        $columns = "s.".$step_table_columns[0];
        for ($i = 1; $i < count($step_table_columns); $i++) {
            $columns .= ", s.".$step_table_columns[$i];
        }
        $order = "";
        if (isset($order_by)) {
            $order = " ORDER BY ".implode(" ASC, ", $order_by)." ASC";
        }
        $union_condition = "";
        if (isset($add_union_condition)) {
        foreach ($add_union_condition as $key => $value)
            $union_condition .= " AND u.".$key." = \"".$value."\" ";
        }
        $step_condition = "";
        if (isset($add_step_condition)) {
        foreach ($add_step_condition as $key => $value)
            $step_condition .= " AND s.".$key." = \"".$value."\" ";
        }
        $sql = "SELECT ".$columns." FROM ".$step_table_name." AS s, ".$union_table_name." AS u WHERE u.".$union_table_columns[1]." = s.id ".$union_condition.$step_condition.$order;
        $retval = $this->wpdb->get_results($sql, OBJECT);
        return (empty($retval) ? NULL : $retval);
    }

      /**
     * Get all values of a table
     * @param string $table_name name of the table
     * @param array $column_condition (facultatif) condition array(column_name => column_value)
     * @param array $order_by (facultatif) list of column name for ascendant order
     * @return array|null array($row_id => array(information_name => information_value))
     * NULL if no result is found
     *
     * @access private
     */
    private function get_all_table($table_name, $column_condition=array(), $order_by = array()) {
        $order = "";
        if ((isset($order_by)) and (!empty($order_by))) {
            $order = " ORDER BY ".implode(" ASC, ", $order_by)." ASC ";
        }
        $condition = "";
        if (!empty($column_condition)) {
            $condition = " WHERE ";
            $conditions=array();
            foreach ($column_condition as $key => $value) {
                $conditions[] = $key." = \"".$value."\" ";
            }
            $condition .= implode(" AND ", $conditions);
        }
        $sql = "SELECT * FROM ".$table_name.$condition." ".$order;
        $table = $this->wpdb->get_results($sql, OBJECT);

        //Tidy output by id
        return $this->tidy_object_table($table, "id");
    }

    /**
      * Prepare the condition array from status
      * @param string $status status of the persons to select (by default $status="actif")
      * @return mixed|null array("status" => $status)
      * empty array if $status is different from "actif" and "alumni"
      * @access private
      */
    private function  prepare_condition_array_from_status($status) {
        if ( $status==='actif' ) {
            return array( "status" => "actif" );
        } elseif ( $status=='alumni' ) {
            return array( "status" => "alumni" );
        } else { // $status='all'
            return  array();
        }
    }

    /*******************************************************************
     * Persons get Methods
    *******************************************************************/

    /**
     * Get person information
     * @param int $person_id id of the person
     * @return array|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_person_information($person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        return $this->get_table_row("MonLabo_persons", $person_id);
    }

    /**
     * Get person information from wp_post_id
     * @param int $wp_post_id id of the page
     * @return mixed|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_person_information_from_wpPostId($wp_post_id) {
        if($this->trigger_error_if_non_numeric($wp_post_id)) {return NULL;}
        $sql = $this->wpdb->prepare("SELECT * FROM MonLabo_persons p WHERE (p.`wp_post_id` REGEXP '(,|^) *(%d) *(,|$)')", $wp_post_id);
        $retval = $this->wpdb->get_row($sql);
        return (empty($retval) ? NULL : $retval);
    }

    /**
     * Get persons information (get all the table of user information)
     * @param int $status actif or alumni
     * @return array|null array($person_id => array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_persons_information( $status = 'actif' ) {
        return $this->get_all_table("MonLabo_persons",
                                    $this->prepare_condition_array_from_status($status),
                                    $order_by = array("last_name", "first_name"));
    }

    /**
     * Get persons information of a team
     * @param int $team_id id of the team
     * @param int $status actif or alumni
     * @return mixed|null array($person_id=> array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_persons_information_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $all_persons = $this->get_step_table_columns(
                            $union_table_name    = "MonLabo_teams_members",
                            $step_table_name     = "MonLabo_persons",
                            $union_table_columns = array("id_team", "id_person"),
                            $step_table_columns  = array("*"),
                            $team_id, $order_by            = array("last_name", "first_name"),
                            $add_union_condition = array(),
                            $add_step_condition = $this->prepare_condition_array_from_status($status)
                        );
        //Tidy output by id
        return $this->tidy_object_table($all_persons, "id");
    }

    /**
     * Get persons information of an unit
     * @param int $unit_id id of the unit
     * @param int $status actif or alumni
     * @return mixed|null array($person_id=> array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_persons_information_for_an_unit($unit_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        $teams_id = $this->get_teams_id_for_an_unit($unit_id);
        $all_persons = array();
        if (isset($teams_id)) {
            foreach ($teams_id as $team_id) {
                $persons = $this->get_persons_information_for_a_team($team_id, $status);
                if (isset($persons)) {
                    foreach ($persons as $person_id => $person_information) {
                        $all_persons[$person_id]=$person_information;
                    }
                }
            }
        }
        return (empty($all_persons) ? NULL : $all_persons);
    }

    /**
     * Get persons information of a category
     * @param int $category category of the person
     * @param string $status status of the persons to select (by default $status="actif")
     * @return mixed|null array($person_id=> array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_persons_information_for_a_category($category, $status="actif"){
        if($this->trigger_error_if_not_set($category)) {return NULL;}
        $add_condition = $this->prepare_condition_array_from_status($status);
        $add_condition["category"]=$category;
        return $this->get_all_table("MonLabo_persons",
                                    $add_condition, $order_by = array("last_name", "first_name"));
    }

    /**
     * Get persons information of a category in a unit
     * @param int $category category of the person
     * @param int $unit_id unit id of the person
     * @param string $status status of the persons to select (by default $status="actif")
     * @return mixed|null array($person_id=> array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_persons_information_for_a_category_in_an_unit($category, $unit_id, $status="actif"){
        if($this->trigger_error_if_not_set($category)) {return NULL;}
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}

        $teams = $this->get_teams_id_for_an_unit($unit_id);
        $persons = array();
        if (isset($teams)) {
            foreach ($teams as $team_id => $id_team) {
                $persons_in_a_team=$this->get_persons_information_for_a_category_in_a_team($category, $team_id, $status);
                if (isset($persons_in_a_team)) {
                    foreach ($persons_in_a_team as $id=>$person_in_a_team) {
                        if(property_exists($person_in_a_team,"id")) {
                            $persons[$person_in_a_team->id] = $person_in_a_team;
                        }
                    }
                }
            }
        }
        return (empty($persons) ? NULL : $persons);
    }

    /**
     * Get persons information of a category in a team
     * @param int $category category of the person
     * @param int $team_id team ID of the person
     * @param string $status status of the persons to select (by default $status="actif")
     * @return mixed|null array($person_id=> array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_persons_information_for_a_category_in_a_team($category, $team_id, $status="actif") {
        if($this->trigger_error_if_not_set($category)) {return NULL;}
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $add_step_condition = $this->prepare_condition_array_from_status($status);
        $add_step_condition["category"]=$category;
        $all_persons = $this->get_step_table_columns(
                                $union_table_name = "MonLabo_teams_members",
                                $step_table_name = "MonLabo_persons",
                                $union_table_columns = array("id_team", "id_person"),
                                $step_table_columns  = array("*"),
                                $team_id, $order_by            = array("last_name", "first_name"),
                                $add_union_condition = array(),
                                $add_step_condition);
        //Tidy output by id
        return $this->tidy_object_table($all_persons, "id");
    }

    /**
     * Get persons name for a team
     * @param int $team_id id of the team
     * @param string $status status of the persons to select (by default $status="actif")
     * @return array|null array(person_id => (person_first_name PERSON_LAST_NAME))
     * NULL if no result is found
     */
    public function get_persons_name_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $persons = $this->get_step_table_columns(
                            $union_table_name    = "MonLabo_teams_members",
                            $step_table_name     = "MonLabo_persons",
                            $union_table_columns = array("id_team", "id_person"),
                            $step_table_columns  = array("id", "first_name", "last_name"),
                            $team_id, $order_by            = array("last_name", "first_name"),
                            $add_union_condition = array(),
                            $add_step_condition = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_for_name($persons, "id");
    }

    /**
     * Get persons id for a team
     * @param int $team_id id of the team
     * @param string $status status of the persons to select (by default $status="actif")
     * @return array|null array(person_id => person_id)
     * NULL if no result is found
     */
    public function get_persons_id_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $persons = $this->get_step_table_columns(
                            $union_table_name    = "MonLabo_teams_members",
                            $step_table_name     = "MonLabo_persons",
                            $union_table_columns = array("id_team", "id_person"),
                            $step_table_columns  = array("id"),
                            $team_id, $order_by = array("id"),
                            $add_union_condition = array(),
                            $add_step_condition = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_and_leave_only_one_column($persons, "id");
    }

    /**
     * Get person name
     * @param int $person_id id of the person
     * @return string|null person name (first_name LAST_NAME)
     * NULL if no result is found
     */
    public function get_person_name($person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $table_columns = array("first_name", "last_name");
        $retval = $this->get_table_variables_from_id("MonLabo_persons", $person_id, $table_columns);
        if (isset($retval)) {
            return $retval->first_name . " ". mb_strtoupper($retval->last_name, 'UTF-8');
        } else {
            return NULL;
        }
    }

    /**
     * Get all persons name
     * @param string $status status of the persons to select (by default $status="actif")
     * @return array|null array(wp_post_id => (first_name LAST_NAME))
     * NULL if no result is found
     */
    public function get_persons_name($status="actif") {
        $persons = $this->get_table_column("MonLabo_persons",
                                            $table_column = array("id", "first_name", "last_name"),
                                            $order_by     = array("last_name", "first_name"),
                                            $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_for_name($persons, "id");
    }

    /*******************************************************************
     * Special persons get Methods
    ******************************************************************/

    /**
     * Get mentors name for a person
     * @param int $person_id id of the person
     * @param string $status status of the mentors to select (by default $status="actif")
     * @return array|null array(mentor_id => (mentor_first_name mentor_LAST_NAME))
     * NULL if no result is found
     */
    public function get_mentors_name_for_a_person($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $mentors = $this->get_step_table_columns(
                                $union_table_name = "MonLabo_mentors",
                                $step_table_name = "MonLabo_persons",
                                $union_table_columns = array("id_person_student", "id_person_supervisor"),
                                $step_table_columns  = array("id", "first_name", "last_name"),
                                $person_id,   $order_by            = array("last_name", "first_name"),
                                $add_union_condition = array(),
                                $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_for_name($mentors, "id");
    }

    /**
     * Get mentors id for a person
     * @param int $person_id id of the person
     * @param string $status status of the mentors to select (by default $status="actif")
     * @return array|null array(mentor_id => mentor_id)
     * NULL if no result is found
     */
    public function get_mentors_id_for_a_person($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $mentors = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_mentors",
                            $step_table_name = "MonLabo_persons",
                            $union_table_columns = array("id_person_student", "id_person_supervisor"),
                            $step_table_columns  = array("id"),
                            $person_id, $order_by            = array("id"),
                            $add_union_condition = array(),
                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_and_leave_only_one_column($mentors, "id");
    }

    /**
     * Get mentors information for a person
     * @param int $person_id id of the person
     * @param string $status status of the students to select (by default $status="actif")
     * @return mixed|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_mentors_information_for_a_person($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $mentors_information = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_mentors",
                            $step_table_name = "MonLabo_persons",
                            $union_table_columns = array("id_person_student", "id_person_supervisor"),
                            $step_table_columns  = array("*"),
                            $person_id, $order_by            = array("last_name", "first_name"),
                            $add_union_condition = array(),
                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table($mentors_information, "id");
    }

    /**
     * Get students name for a person
     * @param int $person_id id of the person
     * @param string $status status of the students to select (by default $status="actif")
     * @return array|null array(student_id => (student_first_name student_LAST_NAME))
     * NULL if no result is found
     */
    public function get_students_name_for_a_person($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $students = $this->get_step_table_columns(
                                $union_table_name = "MonLabo_mentors",
                                $step_table_name = "MonLabo_persons",
                                $union_table_columns = array("id_person_supervisor", "id_person_student"),
                                $step_table_columns  = array("id", "first_name", "last_name"),
                                $person_id,  $order_by            = array("last_name", "first_name"),
                                $add_union_condition = array(),
                                $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_for_name($students, "id");
    }

    /**
     * Get students id for a person
     * @param int $person_id id of the person
     * @param string $status status of the students to select (by default $status="actif")
     * @return array|null array(student_id => student_id)
     * NULL if no result is found
     */
    public function get_students_id_for_a_person($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $students = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_mentors",
                            $step_table_name = "MonLabo_persons",
                            $union_table_columns = array("id_person_supervisor", "id_person_student"),
                            $step_table_columns  = array("id"),
                            $person_id,  $order_by            = array("id"),
                            $add_union_condition = array(),
                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_and_leave_only_one_column($students, "id");
    }

    /**
     * Get students information for a person
     * @param int $person_id id of the person
     * @param string $status status of the students to select (by default $status="actif")
     * @return mixed|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_students_information_for_a_person($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $students_information = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_mentors",
                            $step_table_name = "MonLabo_persons",
                            $union_table_columns = array("id_person_supervisor", "id_person_student"),
                            $step_table_columns  = array("*"),
                            $person_id, $order_by            = array("last_name", "first_name"),
                            $add_union_condition = array(),
                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table($students_information, "id");
    }

    /**
     * Get leaders name for a team
     * @param int $team_id id of the team
     * @param string $status status of the leaders to select (by default $status="actif")
     * @return array|null array(leader_id => (leader_first_name LEADER_LAST_NAME))
     * NULL if no result is found
     */
    public function get_leaders_name_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $leaders = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_teams_members",
                            $step_table_name = "MonLabo_persons",
                            $union_table_columns = array("id_team", "id_person"),
                            $step_table_columns  = array("id", "first_name", "last_name"),
                            $team_id,   $order_by            = array("last_name", "first_name"),
                            $add_union_condition = array("directing" => "1"),
                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_for_name($leaders, "id");
    }

    /**
     * Get leaders id for a team
     * @param int $team_id id of the team
     * @param string $status status of the leaders to select (by default $status="actif")
     * @return array|null array(leader_id => leader_id)
     * NULL if no result is found
     */
    public function get_leaders_id_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $leaders = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_teams_members",
                            $step_table_name = "MonLabo_persons",
                            $union_table_columns = array("id_team", "id_person"),
                            $step_table_columns  = array("id"),
                            $team_id,  $order_by            = array("id"),
                            $add_union_condition = array("directing" => "1"),
                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_and_leave_only_one_column($leaders, "id");
    }

    /**
     * Get leaders information for a team
     * @param int $team_id id of the team
     * @param string $status status of the persons to select (by default $status="actif")
     * @return mixed|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_leaders_information_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $leaders_information = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_members",
                                        $step_table_name  = "MonLabo_persons",
                                        $union_table_columns = array("id_team", "id_person"),
                                        $step_table_columns  = array("*"),
                                        $team_id,  $order_by            = array("last_name", "first_name"),
                                        $add_union_condition = array("directing" => "1"),
                                        $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table($leaders_information, "id");
    }

    /**
     * Get non leaders information for a team
     *
     * @param int $team_id id of the team
     * @param string $status status of the persons to select (by default $status="actif")
     *
     * @return mixed|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_non_leaders_information_for_a_team($team_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $non_leaders_information = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_members",
                                        $step_table_name  = "MonLabo_persons",
                                        $union_table_columns = array("id_team", "id_person"),
                                        $step_table_columns  = array("*"),
                                        $team_id, $order_by            = array("last_name", "first_name"),
                                        $add_union_condition = array("directing" => "0"),
                                        $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table($non_leaders_information, "id");
    }

    /**
     * Get directors information
     * @param string $status status of the persons to select (by default $status="actif")
     * @return mixed|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_directors_information($status="actif") {
        $directors_information = $this->get_all_step_table_columns(
                                            $union_table_name = "MonLabo_units_directors",
                                            $step_table_name = "MonLabo_persons",
                                            $union_table_columns = array("id_unit", "id_person"),
                                            $step_table_columns = array("*"),
                                            $order_by            = array("last_name", "first_name"),
                                            $add_union_condition = array(),
                                            $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table($directors_information, "id");
    }

     /**
      * Get directors name for an unit
      * @param int $unit_id id of the unit
      * @param string $status status of the directors to select (by default $status="actif")
      * @return array|null array(director_first_name DIRECTOR_LAST_NAME)
      * NULL if no result is found
      */
     public function get_directors_name_for_an_unit($unit_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        $directors = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_units_directors",
                                        $step_table_name  = "MonLabo_persons",
                                        $union_table_columns = array("id_unit", "id_person"),
                                        $step_table_columns  = array("id", "first_name", "last_name"),
                                        $unit_id, $order_by            = array("last_name", "first_name"),
                                        $add_union_condition = array(),
                                        $add_step_condition  = $this->prepare_condition_array_from_status($status));
        return $this->tidy_object_table_for_name($directors, "id");
    }

    /**
      * Get directors information for an unit
      * @param int $unit_id id of the unit
      * @param string $status status of the persons to select (by default $status="actif")
      * @return mixed|null array(information_name => information_value)
      * NULL if no result is found
      */
    public function get_directors_information_for_an_unit($unit_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        $directors_information = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_units_directors",
                                        $step_table_name  = "MonLabo_persons",
                                        $union_table_columns = array("id_unit", "id_person"),
                                        $step_table_columns  = array("*"),
                                        $unit_id,  $order_by            = array("last_name", "first_name"),
                                        $add_union_condition = array(),
                                        $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table($directors_information, "id");
    }

    /**
     * Get directors id for an unit
     * @param int $unit_id id of the unit
     * @param string $status status of the directors to select (by default $status="actif")
     * @return array|null array(director_id => (director_first_name DIRECTOR_LAST_NAME))
     * NULL if no result is found
     * @since 1.0.0
     */
    public function get_directors_id_for_an_unit($unit_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        $directors = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_units_directors",
                                        $step_table_name  = "MonLabo_persons",
                                        $union_table_columns = array("id_unit", "id_person"),
                                        $step_table_columns  = array("id"),
                                        $unit_id, $order_by            = array("id_unit"),
                                        $add_union_condition = array(),
                                        $add_step_condition  = $this->prepare_condition_array_from_status($status));
        //Tidy output by id
        return $this->tidy_object_table_and_leave_only_one_column($directors, "id");
    }

    /**
     * Get units id for a director
     * @param int $person_id id of the director
     * @param string $status status of the director to select (by default $status="actif")
     * @return array|null array(unit_id => unit_id)
     * NULL if no result is found
     */
    public function get_units_id_for_a_director($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}

        if ( $status==='actif' ) {
            //$add_step_condition = array( "status" => "actif" );
            $status_string="AND MonLabo_persons.status = \"actif\" ";
        } elseif ( $status==='alumni' ) {
            //$add_step_condition = array( "status" => "alumni" );
            $status_string="AND MonLabo_persons.status = \"alumni\" ";
        } else { // $status='all'
            //$add_step_condition = array();
            $status_string="";
        }
        $sql = $this->wpdb->prepare("SELECT MonLabo_units_directors.id_unit FROM MonLabo_units_directors "
            ."LEFT JOIN MonLabo_persons ON MonLabo_units_directors.id_person = MonLabo_persons.id "
            ." WHERE MonLabo_units_directors.id_person = \"%d\" "
            .$status_string
            ."ORDER BY id_unit ASC", $person_id);
        $units = $this->wpdb->get_results($sql, OBJECT);
        //Tidy output by id
        return $this->tidy_object_table_and_leave_only_one_column($units, "id_unit");
    }

    /*******************************************************************
     * Get teams Methods
    ******************************************************************/

    /**
     * Get team information
     * @param int $team_id id of the team
     * @return array|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_team_information($team_id) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        return $this->get_table_row("MonLabo_teams", $team_id);
    }

    /**
     * Get teams information (get all the table of team information)
     * @return array|null array($team_id=> array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_teams_information(){
        $order_by = array("name_fr", "id_unit");
        $all_teams = $this->get_all_table("MonLabo_teams", array(), $order_by);
        if (!empty($all_teams)) {
            asort($all_teams); //sort teams by order of enter in database.
        }
        return $all_teams;
    }

    /**
     * Get teams information (get all the table of team information)
     * @param int $id_unite id of the unit
     * @return mixed|null array($team_id=>
     *                                      array(information_name => information_value)
     *                         )
     * NULL if no result is found
     */
    public function get_teams_information_for_an_unit($id_unit) {
        if($this->trigger_error_if_non_numeric($id_unit)) {return NULL;}

        $order_by = array("unit", "id");
        $table_column = array("*");
        $column_condition = array("id_unit" => $id_unit);
        return $this->get_table_variables_from_column("MonLabo_teams", $table_column, $column_condition);
    }

    /**
     * Get teams id
     * @param int $id_unite id of the unit
     * @return mixed|null array($team_id => array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_teams_id_for_an_unit($id_unit) {
        if($this->trigger_error_if_non_numeric($id_unit)) {return NULL;}

        $order_by = array("unit", "id");
        $table_column = array("id");
        $column_condition = array("id_unit" => $id_unit);
        $teams = $this->get_table_variables_from_column("MonLabo_teams", $table_column, $column_condition);
        return $this->tidy_object_table_and_leave_only_one_column($teams, "id");
    }


    /**
     * Get team name
     * @param int $team_id id of the team
     * @param string $lang language of the team name (fr or en)
     * @return string|null team name
     * NULL if no result is found
     */
    public function get_team_name($team_id, $lang="fr") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $retval = $this->get_table_variable_from_id("MonLabo_teams", $team_id, $this->column_name_for_language($lang));
        return $retval;
    }

    /**
     * Get all teams name
     * @param string $lang language of the teams name (fr or en)
     * @return array|null array(team_id => team_name)
     * NULL if no result is found
     */
    public function get_teams_name($lang="fr") {
        $column=$this->column_name_for_language($lang);
        $table_column = array("id", $column);
        $order_by = array($column);
        $teams = $this->get_table_column("MonLabo_teams", $table_column, $order_by);
        return $this->tidy_object_table_and_leave_only_one_column($teams, "id", $column);
    }

    /**
     * Get teams name for a person
     * @param int $person_id id of the person
     * @param string $lang language of the thematic name (fr or en)
     * @return array|null array(team_id => team_name)
     * NULL if no result is found
     */
    public function get_teams_name_for_a_person($person_id, $lang="fr") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $column=$this->column_name_for_language($lang);
        $teams = $this->get_step_table_columns(
                            $union_table_name       = "MonLabo_teams_members",
                            $step_table_name        = "MonLabo_teams",
                            $union_table_columns    = array("id_person", "id_team"),
                            $step_table_columns     = array("id", $column),
                            $person_id, $order_by               = array($column));
        return $this->tidy_object_table_and_leave_only_one_column($teams, "id", $column);
    }

    /**
     * Get teams id for a person
     * @param int $person_id id of the person
     * @return array|null array(team_id => team_id)
     * NULL if no result is found
     */
    public function get_teams_id_for_a_person($person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $teams = $this->get_step_table_columns(
                            $union_table_name = "MonLabo_teams_members",
                            $step_table_name  = "MonLabo_teams",
                            $union_table_columns = array("id_person", "id_team"),
                            $step_table_columns  = array("id"),
                            $person_id,  $order_by            = array("id"));
        return $this->tidy_object_table_and_leave_only_one_column($teams, "id");
    }

    /**
     * Get teams id for a person
     * @param int $person_id id of the person
     * @return mixed|null array(team_id => team_id)
     * NULL if no result is found
     */
    public function get_teams_information_for_a_person($person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $teams_information = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_members",
                                        $step_table_name  = "MonLabo_teams",
                                        $union_table_columns = array("id_person", "id_team"),
                                        $step_table_columns  = array("*"),
                                        $person_id,  $order_by            = array("name_fr"));
        //Tidy output by id
        return $this->tidy_object_table($teams_information, "id");
    }

    /**
     * Get team id for a wp_post_id
     * @param int $wp_post_id id of the page
     * @return int|null team_id
     * NULL if no result is found
     */
    public function get_team_id_from_wpPostId($wp_post_id) {
        $sql = $this->wpdb->prepare("SELECT id FROM MonLabo_teams p WHERE (p.`wp_post_id` REGEXP CONCAT('(,|^) *', %s ,' *(,|$)'))", preg_quote($wp_post_id));
        return MonLaboLib::first_key_of_array($this->wpdb->get_results($sql, OBJECT_K));
    }

    /**
     * Get unit id for a wp_post_id
     * @param int $wp_post_id id of the page
     * @return int|null team_id
     * NULL if no result is found
     */
    public function get_unit_id_from_wpPostId($wp_post_id) {
        $sql = $this->wpdb->prepare("SELECT id FROM MonLabo_units p WHERE (p.`wp_post_id` REGEXP CONCAT('(,|^) *', %s ,' *(,|$)'))", preg_quote($wp_post_id));
        return MonLaboLib::first_key_of_array($this->wpdb->get_results($sql, OBJECT_K));
    }

    /**
     * Get person id for a wp_post_id
     * @param int $wp_post_id id of the page
     * @return int|null team_id
     * NULL if no result is found
     */
    public function get_person_id_from_wpPostId($wp_post_id) {
        $sql = $this->wpdb->prepare("SELECT id FROM MonLabo_persons p WHERE (p.`wp_post_id` REGEXP CONCAT('(,|^) *', %s ,' *(,|$)'))", preg_quote($wp_post_id));
        return MonLaboLib::first_key_of_array($this->wpdb->get_results($sql, OBJECT_K));
    }

    /**
     * Get teams information for a thematic
     * @param int $thematic_id id of the thematic
     * @return mixed|null array($team_id => array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_teams_information_for_a_thematic($thematic_id) {
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}
        $teams_information = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_thematics",
                                        $step_table_name  = "MonLabo_teams",
                                        $union_table_columns = array("id_thematic", "id_team"),
                                        $step_table_columns  = array("*"),
                                        $thematic_id,  $order_by            = array("name_fr"));
        //Tidy output by id
        return $this->tidy_object_table($teams_information, "id");
    }

    /**
     * Get teams id for a thematic
     * @param int $thematic_id id of the thematic
     * @return mixed|null array(team_id => team_id)
     * NULL if no result is found
     */
    public function get_teams_id_for_a_thematic($thematic_id) {
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}
        $teams = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_thematics",
                                        $step_table_name  = "MonLabo_teams",
                                        $union_table_columns = array("id_thematic", "id_team"),
                                        $step_table_columns  = array("id"),
                                        $thematic_id,  $order_by            = array("name_fr"));
        //Tidy output by id
        return $this->tidy_object_table_and_leave_only_one_column($teams, "id");
    }

    /**
     * Get teams id for a leader
     * @param int $person_id id of the team_leader
     * @param string $status status of the team leader to select (by default $status="actif")
     * @return array|null array(unit_id => unit_id)
     * NULL if no result is found
     */
    public function get_teams_id_for_a_leader($person_id, $status="actif") {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}

        if ( $status==='actif' ) {
            //$add_step_condition = array( "status" => "actif" );
            $status_string="AND MonLabo_persons.status = \"actif\" ";
        } elseif ( $status==='alumni' ) {
            //$add_step_condition = array( "status" => "alumni" );
            $status_string="AND MonLabo_persons.status = \"alumni\" ";
        } else { // $status='all'
            //$add_step_condition = array();
            $status_string="";
        }
        $sql = $this->wpdb->prepare("SELECT MonLabo_teams_members.id_team FROM MonLabo_teams_members "
            ."LEFT JOIN MonLabo_persons ON MonLabo_teams_members.id_person = MonLabo_persons.id "
            ." WHERE MonLabo_teams_members.id_person = \"%d\" AND MonLabo_teams_members.directing = 1 "
            .$status_string
            ."ORDER BY id_team ASC", $person_id);
        $teams = $this->wpdb->get_results($sql, OBJECT);
        //Tidy output by id
        return $this->tidy_object_table_and_leave_only_one_column($teams, "id_team");
    }

    /*******************************************************************
     * Get thematics Methods
    ******************************************************************/
    /**
     * Get thematic information
     * @param int $thematic_id id of the thematic
     * @return array|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_thematic_information($thematic_id) {
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}
        return $this->get_table_row("MonLabo_thematics", $thematic_id);
    }

    /**
     * Get thematics information (get all the table of thematic information)
     *
     * @return array|null array($team_id => array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_thematics_information() {
        $order_by = array("name_fr");
        return $this->get_all_table("MonLabo_thematics", array(), $order_by);
    }

    /**
     * Get thematic name
     * @param int $thematic_id id of the thematic
     * @param string $lang language of the thematic name (fr or en)
     * @return string|null thematic name
     * NULL if no result is found
     */
    public function get_thematic_name($thematic_id, $lang="fr") {
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}
        return $this->get_table_variable_from_id("MonLabo_thematics", $thematic_id, $this->column_name_for_language($lang));
    }

    /**
     * Get all thematics name
     * @param string $lang language of the thematics name (fr or en)
     * @return array|null array(thematic_id => thematic_name)
     * NULL if no result is found
     */
    public function get_thematics_name($lang="fr") {
        $column=$this->column_name_for_language($lang);
        $table_column = array("id", $column);
        $order_by = array($column);
        $thematics = $this->get_table_column("MonLabo_thematics", $table_column, $order_by);
        return $this->tidy_object_table_and_leave_only_one_column($thematics, "id", $column);
    }

    /**
     * Get thematics name for a team
     * @param int $team_id id of the team
     * @param string $lang language of the thematic name (fr or en)
     * @return array|null array(thematic_id => thematic_name)
     * NULL if no result is found
     */
    public function get_thematics_name_for_a_team($team_id, $lang="fr") {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $column=$this->column_name_for_language($lang);
        $thematics = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_thematics",
                                        $step_table_name  = "MonLabo_thematics",
                                        $union_table_columns = array("id_team", "id_thematic"),
                                        $step_table_columns  = array("id", $column),
                                        $team_id,  $order_by           = array($column));
        //Tidy output by id
        return $this->tidy_object_table_and_leave_only_one_column($thematics, "id", $column);
    }

    /**
     * Get thematics information for a team
     * @param int $team_id id of the team
     * @return mixed|null array(thematic_id => (thematic_information_name => thematic_information_value))
     * NULL if no result is found
     */
    public function get_thematics_information_for_a_team( $team_id ) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $thematics_information = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_thematics",
                                        $step_table_name  = "MonLabo_thematics",
                                        $union_table_columns = array("id_team", "id_thematic"),
                                        $step_table_columns  = array("*"),
                                        $team_id, $order_by = array("name_fr"));
        //Tidy output by id
        return $this->tidy_object_table($thematics_information, "id");
    }

    /**
     * Get thematics id for a team
     * @param int $team_id id of the team
     * @return array|null array(thematic_id => thematic_id)
     * NULL if no result is found
     */
    public function get_thematics_id_for_a_team($team_id) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $thematics = $this->get_step_table_columns(
                                        $union_table_name = "MonLabo_teams_thematics",
                                        $step_table_name  = "MonLabo_thematics",
                                        $union_table_columns = array("id_team", "id_thematic"),
                                        $step_table_columns  = array("id"),
                                        $team_id, $order_by = array("id"));
        //Tidy output by id
        return $this->tidy_object_table_and_leave_only_one_column($thematics, "id");
    }

    /*******************************************************************
     * Get units Methods
    ******************************************************************/

    /**
     * Get unit information
     * @param int $unit_id id of the unit
     * @return array|null array(information_name => information_value)
     * NULL if no result is found
     */
    public function get_unit_information($unit_id) {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        return $this->get_table_row("MonLabo_units", $unit_id);
    }

    /**
     * Get units information (get all the table of unit information)
     * @return array|null array($team_id => array(information_name => information_value))
     * NULL if no result is found
     */
    public function get_units_information() {
        return $this->get_all_table("MonLabo_units", array(), $order_by=array("code"));
    }

    /**
     * Get unit name
     * @param int $unit_id id of the unit
     * @param string $lang language of the units name (fr or en)
     * @return mixed|null array(unit_id => unit_name)
     * NULL if no result is found
     */
    public function get_unit_name( $unit_id, $lang="fr" ) {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        return $this->get_table_variable_from_id("MonLabo_units", $unit_id, $this->column_name_for_language($lang));
    }

    /**
     * Get all units name
     * @param string $lang language of the units name (fr or en)
     * @return array|null array(unit_id => unit_name)
     * NULL if no result is found
     */
    public function get_units_name($lang="fr") {
        $column=$this->column_name_for_language($lang);
        $table_column = array("id", $column);
        $order_by = array($column);
        $units = $this->get_table_column("MonLabo_units", $table_column, $order_by);
        return $this->tidy_object_table_and_leave_only_one_column($units, "id", $column);
    }


    /*******************************************************************
     * Other Methods
     ******************************************************************/

    /**
     * Get all already given functions tydied by category
     *
     * @return array array("categoryA" => array(
     *                                            "categoryA | function1_en | function1_fr" => "function1_en | function1_fr"),
     *                                            "categoryA | function2_en | function2_fr" => "function2_en | function2_fr"),
     *                                            ...
     *                                         )
     *                     "categoryB" => array(...),
     *                     ...
     *                    )
     */
    public function get_multilingual_functions_by_category() {
        //------------------------------------------------------------------------------------------------------
        //1 - Récupère les triplets uniques (category, function_en, function_fr) déjà utilisés par des personnes
        //------------------------------------------------------------------------------------------------------
        $sql = "SELECT MonLabo_persons.category,  MonLabo_persons.function_en,  MonLabo_persons.function_fr "
            ."FROM MonLabo_persons "
            ."GROUP BY CONCAT(category, function_en, function_fr) "
            ."ORDER BY category ASC, function_en ASC, function_fr ASC";
        $functions = $this->wpdb->get_results($sql, OBJECT);

        if(isset($functions)) {
            //------------------------------------------------
            //2 - Ranger les fonctions déjà utilisées par catégories
            //------------------------------------------------
            $functions_multilingual = array();
            foreach ($functions as $function) {
                $category_lowercase=mb_strtolower($function->category, 'UTF-8'); //On converti la catégorie en minuscule (pour une compatibilité avec les anciennes versions de MonLabo)
                if(!array_key_exists($category_lowercase, $functions_multilingual)) {
                    $functions_multilingual[$category_lowercase]= array();
                }
                array_push($functions_multilingual[$category_lowercase], $function->function_en.' | '.$function->function_fr);
            }
            //------------------------------------------------
            //3 - Les fusionner avec les fonctions par défaut.
            //------------------------------------------------
            $MonLabo_personsFunctions_default=get_MonLabo_MembersFunctionsByCategory_default();
            foreach ($MonLabo_personsFunctions_default as $category => $functions_for_a_category) {
                if(!array_key_exists($category, $functions_multilingual)) {
                    $functions_multilingual[$category]=array();
                }
                $functions_multilingual[$category]=array_merge($functions_for_a_category, $functions_multilingual[$category]);
                foreach ($functions_multilingual[$category] as $old_key => $value) {
                    $new_key="$category | ".$value;
                    $functions_multilingual[$category][$new_key]=$value;
                    unset($functions_multilingual[$category][$old_key]);
                }
                //$functions_multilingual[$category] = array_combine($functions_multilingual[$category], $functions_multilingual[$category]);
                asort($functions_multilingual[$category]);
            }
        } else {
            $functions_multilingual = $MonLabo_personsFunctions_default;
        }
        return (empty($functions_multilingual) ? NULL : $functions_multilingual);
    }
    ////////////////////////////////////////////////////////////////////
    // MUTATOR METHODS
    ////////////////////////////////////////////////////////////////////

    /*******************************************************************
     * Insert Methods
     ******************************************************************/
     /**
      * Extrat a list array in param and suppress from data
      * @param string $list_name List to extract
      * @param array $data array(information_name => information_value)
      * @access private
      */
      private function extract_list_array_from_data($list_name, &$data) {
         $list=NULL;
         if (array_key_exists($list_name, $data)) {
             $list=$this->input_array_of_ids($data[$list_name]);
             unset($data[$list_name]);
         }
         return $list;
     }
    /**
     * Insert person information
     *
     * @param array $data array(information_name => information_value)
     * where information_value is an array when information_name is teams or tuteur
     * otherwise information_value is a string
     */
    public function insert_person($data) {
        $teams=$this->extract_list_array_from_data("teams", $data);
        $mentors =$this->extract_list_array_from_data("mentors", $data);
        $students =$this->extract_list_array_from_data("students", $data);

        $this->wpdb->insert("MonLabo_persons", $data);
        $person_id = $this->wpdb->insert_id;
        if (is_array($teams)) {
            $this->set_teams_for_a_person($teams, $person_id);
        }
        if (is_array($mentors)) {
            $this->set_mentors_for_a_person($mentors, $person_id);
        }
        if (is_array($students)) {
            $this->set_students_for_a_person($students, $person_id);
        }
        return $person_id;
    }

    /**
     * Insert team information
     *
     * @param mixed $data array(information_name => information_value)
     * where information_value is an array when information_name is leaders or thematiques
     * otherwise information_value is a string
     */
    public function insert_team($data) {
        $leaders=$this->extract_list_array_from_data("leaders", $data);
        $thematics =$this->extract_list_array_from_data("thematics", $data);
        $this->wpdb->insert("MonLabo_teams", $data);
        $team_id = $this->wpdb->insert_id;
        if ($leaders!=NULL) { $this->set_leaders_for_a_team($leaders, $team_id); }
        if ($thematics!=NULL) { $this->set_thematics_for_a_team($thematics, $team_id); }
        return $team_id;
    }

    /**
     * Insert thematic information
     *
     * @param mixed $data array(information_name => information_value)
     */
    public function insert_thematic($data) {
        $this->wpdb->insert("MonLabo_thematics", $data);
        return $this->wpdb->insert_id;
    }

    /**
     * Insert unit information
     *
     * @param mixed $data array(information_name => information_value)s
     */
    public function insert_unit($data) {
        $directors=$this->extract_list_array_from_data("directors", $data);
        $this->wpdb->insert("MonLabo_units", $data);
        $unit_id = $this->wpdb->insert_id;
        if ($directors!=NULL) { $this->set_directors_for_an_unit($directors, $unit_id); }
        return $unit_id;
    }

    /**
     * Add a mentor to a person
     *
     * @param int $new_mentor_id
     * @param int $person_id
     *
     */
    public function add_mentor_to_a_person($new_mentor_id, $person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        if($this->trigger_error_if_non_numeric($new_mentor_id)) {return NULL;}

        //Test that persons are reals
        $person_information = $this->get_person_information($person_id);
        $mentor_information = $this->get_person_information($new_mentor_id);
        if ( (!empty($person_information)) and (!empty($mentor_information))  ) {
            $mentors_id_for_the_person=$this->get_mentors_id_for_a_person($person_id);
            #If not yet done anf mentor is a real person
            if ((empty($mentors_id_for_the_person)) or (!in_array($new_mentor_id, $mentors_id_for_the_person))){
                $data = array('id_person_supervisor' => $new_mentor_id, 'id_person_student' => $person_id);
                $this->wpdb->insert("MonLabo_mentors", $data);
            }
        }
    }

    /**
     * Add a person to a team
     *
     * @param int $person_id
     * @param int $team_id
     *
     */
    public function add_person_to_a_team($person_id, $team_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        //Test if team and person are real
        $person_information = $this->get_person_information($person_id);
        $team_information = $this->get_team_information($team_id);
        if ( (!empty($person_information)) and (!empty($team_information))  ) {
            $teams_id_for_the_person=$this->get_teams_id_for_a_person($person_id);
            if ((empty($teams_id_for_the_person)) or (!in_array($team_id, $teams_id_for_the_person))){
                $data = array('directing'=>0, 'id_person'=>$person_id, 'id_team'=>$team_id);
                $this->wpdb->insert("MonLabo_teams_members", $data);
            }
        }
    }

    /**
     * Add a leader to a team
     *
     * @param int $leader_id ID of the person that is a leader
     * @param int $team_id
     *
     */
    public function add_leader_to_a_team($leader_id, $team_id) {
        if($this->trigger_error_if_non_numeric($leader_id)) {return NULL;}
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}

        //if team and person are reals
        $leader_information = $this->get_person_information($leader_id);
        $team_information = $this->get_team_information($team_id);
        if ( (!empty($leader_information)) and (!empty($team_information)) ){
            $members=$this->get_persons_id_for_a_team($team_id);
            if ( (!empty($members)) and (in_array($leader_id, $members)) ) {
                //Si la personne est déjà membre de l'équipe, la passe en leader
                $data = array( 'directing' => 1);
                $where = array('id_person' => $leader_id, 'id_team' => $team_id);
                $this->wpdb->update( "MonLabo_teams_members", $data, $where);
            } else {
                //Sinon on l'ajoute comme leader à l'équipe
                $data = array('id_person' => $leader_id, 'id_team' => $team_id, 'directing' => 1);
                $this->wpdb->insert( "MonLabo_teams_members", $data);
            }
        }
    }


    /**
     * Add a thematic to a team
     *
     * @param int $thematic_id
     * @param int $team_id
     *
     */
    public function add_thematic_to_a_team($thematic_id, $team_id) {
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}

        //if team and thematic are reals
        $thematic_information = $this->get_thematic_information($thematic_id);
        $team_information = $this->get_team_information($team_id);
        if ( (!empty($thematic_information)) and (!empty($team_information)) ){
            $thematics_id_of_the_team=$this->get_thematics_id_for_a_team($team_id);
            if ( (empty($thematics_id_of_the_team)) or (!in_array($thematic_id, $thematics_id_of_the_team)) ) {
                $data = array('id_thematic' => $thematic_id, 'id_team' => $team_id);
                $this->wpdb->insert( "MonLabo_teams_thematics", $data);
            }
            return true;
        }
        return false;
    }

    /**
     * Add a director to an unit
     *
     * @param int $director_id
     * @param int $unit_id
     *
     */
    public function add_director_to_an_unit($director_id, $unit_id) {
        if($this->trigger_error_if_non_numeric($director_id)) {return NULL;}
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        //if director person and unit are reals
        $director_information = $this->get_person_information($director_id);
        $unit_information = $this->get_unit_information($unit_id);
        if ( (!empty($director_information))
                and ( (!empty($unit_information)) or ($unit_id===MAIN_STRUCT_NO_UNIT) )
            ){
            $director_id_of_the_unit=$this->get_directors_id_for_an_unit($unit_id);
            if ( (empty($director_id_of_the_unit)) or (!in_array($director_id, $director_id_of_the_unit)) ) {
                $data = array('id_person' => $director_id, 'id_unit' => $unit_id);
                $this->wpdb->insert( "MonLabo_units_directors", $data);
            }
        }
    }

    /**
     * remove a director from an unit
     *
     * @param int $director_id
     * @param int $unit_id
     *
     */
    public function remove_director_from_an_unit($director_id, $unit_id) {
        if($this->trigger_error_if_non_numeric($director_id)) {return NULL;}
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}

        //if director person and unit are reals
        $director_information = $this->get_person_information($director_id);
        $unit_information = $this->get_unit_information($unit_id);
        if ( (!empty($director_information))
              and ( (!empty($unit_information)) or ($unit_id===MAIN_STRUCT_NO_UNIT))
            ){
            $directors_id_of_the_unit=$this->get_directors_id_for_an_unit($unit_id);
            if ((!empty($directors_id_of_the_unit)) and (in_array($director_id, $directors_id_of_the_unit))) {
                $this->delete_relations_between_directors_and_a_unit($unit_id, Array($director_id));
            }
        }
    }

    /*******************************************************************
     * Update Methods
     ******************************************************************/

    /**
     * Update person information
     *
     * @param int $person_id
     * @param mixed $data array(information_name => information_value)
     * where information_value is an array when information_name is teams or tuteur
     * otherwise information_value is a string
     */
    public function update_person($person_id, $data) {
        if ((isset($data["id"])) and ((!empty($data["id"])) and ($data["id"]!=$person_id))) {
            $person_id="invalid"; //Trigger an error below
        }
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}

        $person_information = $this->get_person_information($person_id);
        if(!empty($person_information)) {
            if (array_key_exists("id", $data)) { unset($data["id"]); }
            $teams=$this->extract_list_array_from_data("teams", $data);
            $mentors=$this->extract_list_array_from_data("mentors", $data);
            $students=$this->extract_list_array_from_data("students", $data);
            if (!empty($data)) {
                $where = array('id' => $person_id);
                $this->wpdb->update("MonLabo_persons", $data, $where);
            }
            if(($teams!=NULL) or (is_array($teams))) {
                //if $team is not nul enter here
                //if $teams is an empty Array() (considered as NULL)
                // enter here to delete all teams
                $this->set_teams_for_a_person($teams, $person_id);
            };
            if(($mentors!=NULL) or (is_array($mentors))) {
                //if $mentors is not nul enter here
                //if $mentors is an empty Array() (considered as NULL)
                // enter here to delete all mentors
                $this->set_mentors_for_a_person($mentors, $person_id);
            };
            if(($students!=NULL) or (is_array($students))) {
                //if $students is not nul enter here
                //if $students is an empty Array() (considered as NULL)
                // enter here to delete all mentors
                $this->set_students_for_a_person($students, $person_id);
            };

        }
    }

    /**
     * Update team information
     *
     * @param int $team_id
     * @param mixed $data array(information_name => information_value)
     * where information_value is an array when information_name is leaders or thematiques
     * otherwise information_value is a string
     */
    public function update_team($team_id, $data) {
        if ((isset($data["id"])) and ((!empty($data["id"])) and ($data["id"]!=$team_id))) {
            $team_id="invalid"; //trigger error next line
        }
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}

        $team_information = $this->get_team_information($team_id);
        if (!empty($team_information)) {
            if (array_key_exists("id", $data)) { unset($data["id"]); }
            $leaders  =$this->extract_list_array_from_data("leaders", $data);
            $thematics=$this->extract_list_array_from_data("thematics", $data);
            if (!empty($data)) {
                $where = array('id' => $team_id);
                $this->wpdb->update("MonLabo_teams", $data, $where);
            }
            if(($leaders!=NULL) or (is_array($leaders))) {
                //if $leaders is not nul enter here
                //if $leaders is an empty Array() (considered as NULL)
                // enter here to delete all leaders
                $this->set_leaders_for_a_team($leaders, $team_id);
            }
            if(($thematics!=NULL) or (is_array($thematics))) {
                //if $thematics is not nul enter here
                //if $thematics is an empty Array() (considered as NULL)
                // enter here to delete all thematics
                $this->set_thematics_for_a_team($thematics, $team_id);
            };
        }
    }

    /**
     * Update thematic information
     *
     * @param int $thematic_id
     * @param mixed $data array(information_name => information_value)
     */
    public function update_thematic($thematic_id, $data) {
        if ((isset($data["id"])) and ((!empty($data["id"])) and ($data["id"]!=$thematic_id))) {
            $thematic_id = "invalid"; //Trigger an error next line
        }
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}

        $thematic_information = $this->get_thematic_information($thematic_id);
        if (!empty($thematic_information)) {
            if (array_key_exists("id", $data)) {
                unset($data["id"]);
            }
            if (!empty($data)) {
                $where = array('id' => $thematic_id);
                $this->wpdb->update("MonLabo_thematics", $data, $where);
            }
        }
    }

    /**
     * Update unit information
     * @param int $unit_id
     * @param mixed $data array(information_name => information_value)
     * where information_value is an array when information_name is directors
     * otherwise information_value is a string
     */
    public function update_unit($unit_id, $data) {
        if ((isset($data["id"])) and ((!empty($data["id"])) and ($data["id"]!=$unit_id))) {
            $unit_id = "invalid"; //Trigger an error next line
        }
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}

        $unit_information = $this->get_unit_information($unit_id);
        if ((!empty($unit_information)) or ($unit_id===MAIN_STRUCT_NO_UNIT)) {
            if (array_key_exists("id", $data)) {
                unset($data["id"]);
            }
            $directors=$this->extract_list_array_from_data("directors", $data);
            if ((!empty($data)) and ($unit_id!=MAIN_STRUCT_NO_UNIT)) {
                //On ne met pas à jour la structure mère.
                $where = array('id' => $unit_id);
                $this->wpdb->update("MonLabo_units", $data, $where);
            }
            if (($directors!=NULL) or (is_array($directors))) {
                //if $directors is not nul enter here
                //if $directors is an empty Array() (considered as NULL)
                // enter here to delete all directors
                $this->set_directors_for_an_unit($directors, $unit_id);
            }
        }
    }

    /**
     * Update teams for a person
     * @param mixed $new_teams_id array of teams id
     * @param int $person_id
     * @access private
     */
    private function set_teams_for_a_person($new_teams_id, $person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}

        $old_teams_id = $this->get_teams_id_for_a_person($person_id);
        $new_teams_id=$this->input_array_of_ids($new_teams_id);
        if (empty($new_teams_id)) {
            $new_teams_id = array();
        }
        if (empty($old_teams_id)) {
            $old_teams_id = array();
        }
        //On efface toutes les équipes qui ne sont pas dans la nouvelle liste $new_teams_id
        $old_teams_id_to_delete=array_diff($old_teams_id, $new_teams_id);
        if(!empty($old_teams_id_to_delete)) {
            $this->delete_relations_between_teams_and_a_person($person_id, $old_teams_id_to_delete);
        }

        //On rajoute les autres équipes nouvelles
        $new_teams_id_to_add=array_diff($new_teams_id, $old_teams_id);
        if(!empty($new_teams_id_to_add)) {
            foreach ($new_teams_id_to_add as $new_team_id_to_add) {
                //Add only if the new team exists
                $this->add_person_to_a_team($person_id, $new_team_id_to_add);
            }
        }
    }

    /**
     * Set mentors for a person
     * @param mixed $new_mentors_id array of mentors ID
     * @param int $student_id
     * @access private
     */
    private function set_mentors_for_a_person($new_mentors_id, $student_id) {
        if($this->trigger_error_if_non_numeric($student_id)) {return NULL;}

        $old_mentors_id = $this->get_mentors_id_for_a_person($student_id);
        $new_mentors_id=$this->input_array_of_ids($new_mentors_id);
        if (empty($new_mentors_id)) {
            $new_mentors_id = array();
        }
        if (empty($old_mentors_id)) {
            $old_mentors_id = array();
        }
        //On efface tous les tuteurs qui ne sont pas dans la nouvelle liste $new_mentors_id
        $old_mentors_id_to_delete=array_diff($old_mentors_id, $new_mentors_id);
        if(!empty($old_mentors_id_to_delete)) {
            $this->delete_relations_between_a_person_and_his_or_her_mentors($student_id, $old_mentors_id_to_delete);
        }

        //On rajoute les autres tuteurs nouveaux
        $new_mentors_id_to_add=array_diff($new_mentors_id, $old_mentors_id);
        //If person to add exists
        if(!empty($new_mentors_id_to_add)) {
            foreach ($new_mentors_id_to_add as $new_mentor_id_to_add) {
                $this->add_mentor_to_a_person($new_mentor_id_to_add, $student_id);
            }
        }
    }

    /**
     * Set students for a person
     * @param mixed $new_students_id array of students ID
     * @param int $mentor_id
     * @access private
     */
    private function set_students_for_a_person($new_students_id, $mentor_id) {
        if($this->trigger_error_if_non_numeric($mentor_id)) {return NULL;}

        $old_students_id = $this->get_students_id_for_a_person($mentor_id);
        $new_students_id=$this->input_array_of_ids($new_students_id);
        if (empty($new_students_id)) {
            $new_students_id = array();
        }
        if (empty($old_students_id)) {
            $old_students_id = array();
        }
        //On efface toutes les étudiants qui ne sont pas dans la nouvelle liste $new_students_id
        $old_students_id_to_delete=array_diff($old_students_id, $new_students_id);
        if(!empty($old_students_id)) {
            $this->delete_relations_between_a_person_and_his_or_her_students($mentor_id, $old_students_id_to_delete);
        }
        //On rajoute les autres tuteurs nouveaux
        $new_students_id_to_add=array_diff($new_students_id, $old_students_id);
        //If person to add exists
        if(!empty($new_students_id_to_add)) {
            foreach ($new_students_id_to_add as $new_student_id_to_add) {
                $this->add_mentor_to_a_person($mentor_id, $new_student_id_to_add);
            }
        }
    }
    /**
     * Update leaders for a team     *
     * @param mixed $new_leaders_id array of leaders id
     * @param int $team_id
     * @access private
     */
    private function set_leaders_for_a_team($new_leaders_id, $team_id) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}

        $team_information = $this->get_team_information($team_id);
        if (!empty($team_information)) {
            $new_leaders_id=$this->input_array_of_ids($new_leaders_id);
            if (empty($new_leaders_id)) {
                $new_leaders_id = array();
            }
            //On efface tous les leaders de l'équipes
            $this->delete_relations_between_leaders_and_a_team($team_id);
            $persons_id_for_the_team=$this->get_persons_id_for_a_team($team_id);
            foreach ($new_leaders_id as $new_leader_id) {
                //Add only if the person exists
                $leader_information = $this->get_person_information($new_leader_id);
                if (!empty($leader_information)) {
                    if( (!empty($persons_id_for_the_team)) and (in_array($new_leader_id, $persons_id_for_the_team)) ) {
                        $where = array( 'id_person' => $new_leader_id,
                                        'id_team'   => $team_id);
                        $data = array( 'directing' => 1);
                        $this->wpdb->update("MonLabo_teams_members", $data, $where);
                    } else {
                        //Sinon on rajoute la personne comme leader
                        $this->add_leader_to_a_team ($new_leader_id, $team_id);
                    }
                }
            }
        }
    }
    /**
     * Update thematics for a team
     * @param mixed $new_thematics_id array of thematics id
     * @param int $team_id
     * @access private
     */
    private function set_thematics_for_a_team($new_thematics_id, $team_id) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}

        $team_information = $this->get_team_information($team_id);
        if (!empty($team_information)) {
            $old_thematics_id = $this->get_thematics_id_for_a_team($team_id);
            $new_thematics_id=$this->input_array_of_ids($new_thematics_id);
            if (empty($new_thematics_id)) {
                $new_thematics_id = array();
            }
            if (empty($old_thematics_id)) {
                $old_thematics_id = array();
            }
            //On efface toutes les thématiques qui ne sont pas dans la nouvelle liste $new_thematics_id
            $old_thematics_id_to_delete=array_diff($old_thematics_id, $new_thematics_id);
            if(!empty($old_thematics_id_to_delete)) {
                $this->delete_relations_between_thematics_and_a_team($team_id, $old_thematics_id_to_delete);
            }

            //On rajoute les autres thématiques nouvelles
            $new_thematics_id_to_add=array_diff($new_thematics_id, $old_thematics_id);
            if(!empty($new_thematics_id_to_add)) {
                foreach ($new_thematics_id_to_add as $new_thematic_id_to_add) {
                    $this->add_thematic_to_a_team($new_thematic_id_to_add, $team_id);
                }
            }
        }
    }

    /**
     * Update directors for an unit
     * @param mixed $new_directors_id array of directors id
     * @param int $unit_id
     * @access private
     */
    private function set_directors_for_an_unit($new_directors_id, $unit_id) {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}

        $unit_information = $this->get_unit_information($unit_id);
        if ((!empty($unit_information)) or ($unit_id===MAIN_STRUCT_NO_UNIT)) {
            if ( (!empty($new_directors_id)) and (!is_array($new_directors_id))) {
                $new_directors_id=$this->input_array_of_ids($new_directors_id);
            }
            $this->delete_relations_between_directors_and_a_unit($unit_id);
            if(!empty($new_directors_id)) {
                foreach ($new_directors_id as $new_director_id) {
                    $this->add_director_to_an_unit($new_director_id, $unit_id);
                }
            }
        }
    }

    /*******************************************************************
     * Delete Methods
     ******************************************************************/

    /**
     * Delete person information
     * @param int $person_id
     */
    public function delete_person($person_id) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}
        $person_information = $this->get_person_information($person_id);
        if (!empty($person_information)) {
            $this->delete_relations_between_teams_and_a_person($person_id);
            $this->delete_relations_between_a_person_and_his_or_her_mentors($person_id);
            $this->delete_relations_between_units_and_a_director($person_id);
            $this->wpdb->delete("MonLabo_persons", array('id' => $person_id));
        }
    }

    /**
     * Delete team information
     * @param int $team_id
     */
    public function delete_team($team_id) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}
        $team_information = $this->get_team_information($team_id);
        if (!empty($team_information)) {
            $this->delete_relations_between_persons_and_a_team($team_id);
            $this->delete_relations_between_thematics_and_a_team($team_id);
            $this->wpdb->delete("MonLabo_teams", array('id' => $team_id));
        }
    }

    /**
     * Delete thematic information
     * @param int $thematic_id
     */
    public function delete_thematic($thematic_id) {
        if($this->trigger_error_if_non_numeric($thematic_id)) {return NULL;}
        $thematic_information = $this->get_thematic_information($thematic_id);
        if (!empty($thematic_information)) {
            $this->delete_relations_between_teams_and_a_thematic($thematic_id);
            $this->wpdb->delete("MonLabo_thematics", array('id' => $thematic_id));
        }
    }

    /**
     * Delete unit information
     * @param int $unit_id
     */
    public function delete_unit($unit_id) {
        if($this->trigger_error_if_non_numeric($unit_id)) {return NULL;}
        $unit_information = $this->get_unit_information($unit_id);
        if (!empty($unit_information)) {
            //Delete relations between directors and this unit
            $this->delete_relations_between_directors_and_a_unit($unit_id);
            //Delete relations between teams and this unit
            $this->delete_relations_between_teams_and_a_unit($unit_id);
            //Delete unit
            $this->wpdb->delete("MonLabo_units", array('id' => $unit_id));
        }
    }

    /*---------------------------------------
    /*  Delete relation methods
    ----------------------------------------*/
    /**
     * Delete retation between teams and a unit
     * @param int $person_id
     * @param Array $optional_teams_list optional list of teams to unlink
     *             if NULL ==> unlink all teams
     * @access private
     */
    private function delete_relations_between_teams_and_a_unit($unit_id, $optional_teams_list=NULL) {
        if ($optional_teams_list===NULL) {
            $teams_id=$this->get_teams_id_for_an_unit($unit_id);
        } else {
            $teams_id=$optional_teams_list;
        }
        //Delete relations between teams and this unit
        if (isset($teams_id)) {
            foreach ($teams_id as $team_id) {
                $this->update_team($team_id, array("id_unit" => NULL));
            }
        }
    }

    /**
     * Delete teams from a person
     * @param int $person_id
     * @param Array $optional_teams_list optional list of teams to unlink
     *             if NULL ==> unlink all teams
     * @access private
     */
    private function delete_relations_between_teams_and_a_person($person_id, $optional_teams_list=NULL) {
        if ($optional_teams_list===NULL) {
            $teams_id_to_delete = $this->get_teams_id_for_a_person($person_id);
        } else {
            $teams_id_to_delete=$optional_teams_list;
        }
        $this->delete_relations_in_a_table("MonLabo_teams_members", 'id_person', $person_id,
                                                            'id_team',   $teams_id_to_delete);
    }

    /**
     * Delete mentors from a person
     * @param int $person_id
     * @param Array $optional_mentors_list optional list of mentors to unlink
     *             if NULL ==> unlink all mentors
     * @access private
     */
    private function delete_relations_between_a_person_and_his_or_her_mentors($person_id, $optional_mentors_list=NULL) {
        if ($optional_mentors_list===NULL) {
            $mentors_id_to_delete = $this->get_mentors_id_for_a_person($person_id);
        } else {
            $mentors_id_to_delete=$optional_mentors_list;
        }
        $this->delete_relations_in_a_table("MonLabo_mentors", 'id_person_student', $person_id,
                                                            'id_person_supervisor',   $mentors_id_to_delete);
    }

    /**
     * Delete students from a person
     * @param int $person_id
     * @param Array $optional_students_list optional list of students to unlink
     *             if NULL ==> unlink all mentors
     * @access private
     */
    private function delete_relations_between_a_person_and_his_or_her_students($person_id, $optional_students_list=NULL) {
        if ($optional_students_list===NULL) {
            $students_id_to_delete = $this->get_students_id_for_a_person($person_id);
        } else {
            $students_id_to_delete=$optional_students_list;
        }
        $this->delete_relations_in_a_table("MonLabo_mentors", 'id_person_supervisor',   $person_id
                                                            , 'id_person_student', $students_id_to_delete);
    }



    /**
     * Delete persons for a team a team
     * @param int $team_id
     * @param Array $optional_persons_list optional list of persons to unlink
     *             if NULL ==> unlink all persons
     * @access private
     */
    private function delete_relations_between_persons_and_a_team($team_id, $optional_persons_list=NULL) {
        if ($optional_persons_list===NULL) {
            $persons_id_to_delete = $this->get_persons_id_for_a_team($team_id);
        } else {
            $persons_id_to_delete=$optional_persons_list;
        }
        $this->delete_relations_in_a_table("MonLabo_teams_members", 'id_team', $team_id,
                                                            'id_person',   $persons_id_to_delete);
    }

    /**
     * Delete thematics from a team
     * @param int $team_id
     * @param Array $optional_thematics_list optional list of thematics to unlink
     *             if NULL ==> unlink all thematics
     * @access private
     */
    private function delete_relations_between_thematics_and_a_team($team_id, $optional_thematics_list=NULL) {
        if ($optional_thematics_list===NULL) {
            $thematics_for_a_team=$this->get_thematics_id_for_a_team($team_id);
        } else {
            $thematics_for_a_team=$optional_thematics_list;
        }
        $this->delete_relations_in_a_table("MonLabo_teams_thematics", 'id_team', $team_id,
                                                            'id_thematic',   $thematics_for_a_team);
    }

    /**
     * Delete leaders from a team
     * @param int $team_id
     * @param Array $optional_leaders_list optional list of leaders to unlink
     *             if NULL ==> unlink all leaders
     */
    private function delete_relations_between_leaders_and_a_team($team_id, $optional_leaders_list=NULL) {
        if($this->trigger_error_if_non_numeric($team_id)) {return NULL;}

        if ($optional_leaders_list===NULL) {
            $leaders_for_a_team=$this->get_leaders_id_for_a_team($team_id);
        } else {
            $leaders_for_a_team=$optional_leaders_list;
        }
        if(!empty($leaders_for_a_team)) {
            foreach ($leaders_for_a_team as $leader_for_a_team) {
                $where = array( 'id_person' => $leader_for_a_team,
                                'id_team'   => $team_id);
                $data  = array('directing' => 0);
                $this->wpdb->update("MonLabo_teams_members", $data, $where);
            }
        }
    }

    /**
     * Delete teams from an thematic
     * @param int $thematic_id
     * @param Array $optional_teams_list optional list of leaders to unlink
     *             if NULL ==> unlink all leaders
     * @access private
     */
    private function delete_relations_between_teams_and_a_thematic($thematic_id, $optional_teams_list=NULL) {
        if ($optional_teams_list===NULL) {
            $teams_id_for_a_thematic=$this->get_teams_id_for_a_thematic($thematic_id);
        } else {
            $teams_id_for_a_thematic=$optional_teams_list;
        }
        $this->delete_relations_in_a_table("MonLabo_teams_thematics", 'id_thematic', $thematic_id,
                                                            'id_team',   $teams_id_for_a_thematic);
    }

    /**
     * Delete directors from an unit
     * @param int $unit_id
     * @param Array $optional_directors_list optional list of directors to unlink
     *             if NULL ==> unlink all directors
     * @access private
     */
    private function delete_relations_between_directors_and_a_unit($unit_id, $optional_directors_list=NULL) {
        if ($optional_directors_list===NULL) {
            $directors_id_for_a_unit=$this->get_directors_id_for_an_unit($unit_id);
        } else {
            $directors_id_for_a_unit=$optional_directors_list;
        }
        $this->delete_relations_in_a_table("MonLabo_units_directors", 'id_unit', $unit_id,
                                                            'id_person',   $directors_id_for_a_unit);
    }

    /**
     * Delete units from a director
     * @param int $person_id
     * @param Array $optional_units_list optional list of units to unlink
     *             if NULL ==> unlink all units
     * @access private
     */
    private function delete_relations_between_units_and_a_director($person_id, $optional_units_list=NULL) {
        if($this->trigger_error_if_non_numeric($person_id)) {return NULL;}

        if ($optional_units_list===NULL) {
            $units_id_for_a_director = $this->get_units_id_for_a_director($person_id);
        } else {
            $optional_units_list = MonLaboLib::prepare_multiple_values_variable_into_array($optional_units_list);
            $units_id_for_a_director=$optional_units_list;
        }
        $this->delete_relations_in_a_table("MonLabo_units_directors", 'id_person', $person_id,
                                                            'id_unit',   $units_id_for_a_director);
        }


    /*---------------------------------------
    /*  Generic delete methods
    ----------------------------------------*/
    /**
     * Delete in DB table=$table delete all element
     *         where $item1_name = $item1_value
     *         and $item2_name = item2_value
     *  for all item2_value in $item2_list
     *
     * @param string $table
     * @param string $item1_name
     * @param string $table
     * @param string $item2_name
     * @param Array $item2_list list of items to unlink
     *             if NULL ==> unlink all teams
     * @access private
     */
    private function delete_relations_in_a_table($table,    $item1_name, $item1_value,
                                                            $item2_name, $item2_list) {
        if($this->trigger_error_if_non_numeric($item1_value)) {return NULL;}
        if(!empty($item2_list)) {
            foreach ($item2_list as $item2_element) {
                $where = array( $item1_name => $item1_value,
                                $item2_name => $item2_element);
                $this->wpdb->delete($table, $where);
            }
        }
    }
}
?>
