<?php
///////////////////////////////////////////////////////////////////////////////////////////
// Librairie de fonctions pour manipuler les équipes.
///////////////////////////////////////////////////////////////////////////////////////////
/**
 * Class MonLabo_teams
 *
 * @package
 */

/*
    sort_by_name()
    filter_with_categories($categories)
    filter_with_ids($persons_ids)
    filter_with_groups($groups)
    filter_with_units($units)
*/
class MonLabo_teams {

    private $teams_table;

    public function __construct($teams_table=array()) {
        if(!is_array($teams_table)) {$teams_table=array();}
        $this->$teams_table = $teams_table;
    }

    public function get_teams() {
        return $this->$teams_table;
    }

    public function count() {
        return count($this->$teams_table);
    }

    //--------------------------------------------------------------------------------
    // Trie un tableau de $teams_information par nom
    //--------------------------------------------------------------------------------
    public function sort_by_name(){
        $table_to_sort=array();
        if ($this->count()>0) {
            foreach ($this->$teams_table as $key=>$team_info) {
                if (isset($team_info->name_en)) {
                    $table_to_sort["$team_info->name_en".$key ] = $team_info;
                }
            }
            ksort($table_to_sort);
        }
        $this->$teams_table=$table_to_sort;
        return $this;
    }

    //--------------------------------------------------------------------------------
    // Supprime les équipes qui ne sont pas dans l'un des groupes fournis en argument
    //--------------------------------------------------------------------------------
    public function filter_with_groups($groups){
        if (!empty($this->$teams_table)) {
            //1 - On établi une liste d'id d'équipes qui seront autorisées
            $MonLabo_access_data = New MonLabo_access_data();
            if(empty($units)) {
                $this->$teams_table=array();
                $this->team_filter=array();
                return $this;
            } else {
              $thematics_id_for_groups=array();
              foreach ($groups as $group_id) {
                  $thematics_id_for_group=$MonLabo_access_data->get_teams_id_for_a_thematic($group_id);
                  if ($thematics_id_for_group===NULL) { $thematics_id_for_group=Array();}
                  $thematics_id_for_groups=$thematics_id_for_groups+$thematics_id_for_group;
              }
            }
            //2 - On supprime toute équipe n'etant pas dans la liste autorisée
            $this->filter_with_ids($thematics_id_for_groups);
        }
        return $this;
    }

    //--------------------------------------------------------------------------------
    // Supprime les équipes qui ne sont pas dans l'une des unités fournies en argument
    //--------------------------------------------------------------------------------
    public function filter_with_units($units){
        if (!empty($this->$teams_table)) {
            //1 - On établi une liste d'id d'équipes qui seront autorisées
            $MonLabo_access_data = New MonLabo_access_data();
            if(empty($units)) {
                $this->$teams_table=array();
                $this->team_filter=array();
                return $this;
            } else {
              $teams_id_for_units=array();
              foreach ($units as $unit_id) {
                  $teams_id_for_unit=$MonLabo_access_data->get_teams_id_for_an_unit($unit_id);
                  if ($teams_id_for_unit===NULL) { $teams_id_for_unit=Array();}
                  $teams_id_for_units=$teams_id_for_units+$teams_id_for_unit;
              }
            }
            //2 - On supprime toute équipe n'etant pas dans la liste autorisée
            $this->filter_with_ids($teams_id_for_units);
        }
        return $this;
    }
    //--------------------------------------------------------------------------------
    // Supprime les teams qui ne sont pas dans une liste fournie en argument
    //--------------------------------------------------------------------------------
    public function filter_with_ids($teams_ids){
        if (!empty($this->$teams_table)) {

            if(empty($teams_ids)) {
                //On supprime tout
                $this->$teams_table=array();
                return $this;
            }

            //1 - On indexe pour suppression toute team n'etant pas dans la liste autorisée
            $key_to_suppress = array();
            foreach ($this->$teams_table as $key => $team) {
                if (!in_array($key, $teams_ids)) {
                    $key_to_suppress[$key] = $key;
                }
            }

            //2 - On procède à la suppression effective
            if (!empty($key_to_suppress)) {
                foreach ($key_to_suppress as $key => $dummy) {
                    unset($this->$teams_table[$key]);
                }
            }
        }
        return $this;
    }

}
